package es.guti.bsp_guti;

import java.util.UUID;
import java.util.Vector;

import org.bioapi.Archive;
import org.bioapi.BFP;
import org.bioapi.BFPEnumerationListener;
import org.bioapi.BFPEventListener;
import org.bioapi.BFPGUIProgressEventListener;
import org.bioapi.BSP;
import org.bioapi.BioAPIException;
import org.bioapi.Comparison;
import org.bioapi.GUIProgressEventListener;
import org.bioapi.GUISelectEventListener;
import org.bioapi.GUIStateEventListener;
import org.bioapi.Processing;
import org.bioapi.Sensor;
import org.bioapi.data.ACBioParameters;
import org.bioapi.data.BFPListElement;
import org.bioapi.data.BIR;
import org.bioapi.data.BSPSchema;
import org.bioapi.data.Candidate;
import org.bioapi.data.DataFormat;
import org.bioapi.data.DataTypes;
import org.bioapi.data.DataTypes.BIRDatabaseAccess;
import org.bioapi.data.DataTypes.BIRPurpose;
import org.bioapi.data.DataTypes.BiometricSubtype;
import org.bioapi.data.DataTypes.BiometricType;
import org.bioapi.data.DataTypes.Facility;
import org.bioapi.data.DataTypes.GUISuboperation;
import org.bioapi.data.DataTypes.ResultOptions;
import org.bioapi.data.DataTypes.UnitIndicatorStatus;
import org.bioapi.data.DataTypes.UnitPowerMode;
import org.bioapi.data.IdentifyPopulation;
import org.bioapi.data.PopulationMember;
import org.bioapi.data.SecurityProfileType;
import org.bioapi.data.UnitListElement;
import org.bioapi.data.UnitSchema;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.util.SparseIntArray;

/**
 * This code corresponds to a partial implementation of a BSP according to the
 * Beta 4 of ISO/IEC 30106-2 "BioAPI for object oriented programming languages
 * -- Part 2: Java implementation" standard. The main function of this BSP is to
 * provide a reference implementation for biometric identification and
 * enrollment.
 * 
 * @author Daniel Sierra-Ramos
 */
public class BSPGUTI implements BSP {

	// BSP_UUID
	private final UUID BSP_UUID = UUID
			.fromString("00000000-0000-0000-0000-000000000001");

	// REFERENCE UNITS
	private Sensor sensor;
	private Archive archive;
	private Processing processing;
	private Comparison comparison;

	private final boolean ARCHIVE_SUPPORTED = true;
	private final boolean SENSOR_SUPPORTED = true;
	private final boolean PROCESSING_SUPPORTED = true;
	private final boolean COMPARISON_SUPPORTED = true;

	private final int DONT_CARE = 0x00000000;
	private final int DONT_INCLUDE = 0xFFFFFFFF;

	// Private lists to store the compatible units reference to this BSP
	private Vector<Sensor> compatibleSensorUnits = new Vector<Sensor>();
	private Vector<UnitSchema> compatibleSensorSchemas = new Vector<UnitSchema>();
	private Vector<Archive> compatibleArchiveUnits = new Vector<Archive>();
	private Vector<UnitSchema> compatibleArchiveSchemas = new Vector<UnitSchema>();
	private Vector<Processing> compatibleProcessingUnits = new Vector<Processing>();
	private Vector<UnitSchema> compatibleProcessingSchemas = new Vector<UnitSchema>();
	private Vector<Comparison> compatibleComparisonUnits = new Vector<Comparison>();
	private Vector<UnitSchema> compatibleComparisonSchemas = new Vector<UnitSchema>();
	private Vector<UnitSchema> compatibleUnits = new Vector<UnitSchema>();
	private Vector<BFP> loadedBFPs;

	private int accumulatedUnitID = 0;
	private SparseIntArray unitIDs = new SparseIntArray();

	private BSPSchema bspSchema;
	private BIR biometricReference;
	private byte[] acBioInstance;

	// Callback interfaces
	private GUISelectEventListener selectHandler;
	private GUIStateEventListener stateHandler;
	private GUIProgressEventListener progressHandler;

	Context applicationContext;

	boolean timeoutSwitch = false;
	boolean bspLoaded = false;

	private final String UNSUPPORTED_MESSAGE = "This method is still under revision";

	public BSPGUTI(Context applicationContext) throws BioAPIException {
		this.bspSchema = createBSPSchema();
		this.applicationContext = applicationContext;
	}

	public BSPGUTI(BSPSchema bspSchema, Context applicationContext)
			throws BioAPIException {
		this.bspSchema = bspSchema;

		compatibleUnits = new Vector<UnitSchema>();
		this.applicationContext = applicationContext;
	}

	private BSPSchema createBSPSchema() {
		// BSP schema parameters
		final String BSP_DESCRIPTION = "BSP-GUTI";
		final String PATH = null;
		final String SPEC_VERSION = "demo1.0";
		final String PRODUCT_VERSION = "demo1.0";
		final String VENDOR = "GUTI";
		final Vector<DataFormat> BSP_SUPPORTED_FORMATS = null;
		final Vector<DataTypes.BiometricType> FACTORS_MASK = new Vector<DataTypes.BiometricType>();
		final Vector<DataTypes.BSPSchemaOperations> OPERATIONS = new Vector<DataTypes.BSPSchemaOperations>();
		final Vector<DataTypes.BSPSchemaOptions> OPTIONS = new Vector<DataTypes.BSPSchemaOptions>();
		final int PAYLOAD_POLICY = 0;
		final int MAX_PAYLOAD_SIZE = 5000;
		final int DEFAULT_VERIFY_TIMEOUT = 0;
		final int DEFAULT_IDENTIFY_TIMEOUT = 0;
		final int DEFAULT_CAPTURE_TIMEOUT = 5000;
		final int DEFAULT_ENROL_TIMEOUT = 10000;
		final int DEFAUT_CALIBRATE_TIMEOUT = 0;
		final int MAX_BSP_DB_SIZE = 0;
		final int MAX_IDENTIFY = 0xFFFFFFFF;
		final int MAX_NUM_ENROL_INSTANCES = 1;
		final byte[] HOSTING_ENDPOINT_IRI = null;
		final UUID BSP_ACCESS_UUID = UUID.randomUUID();
		final Vector<DataFormat> BSP_SUPPORTED_ALGORITHMS = null;
		final Vector<UUID> BSP_SUPPORTED_TRANSFORM_OPERATIONS = null;

		FACTORS_MASK.add(BiometricType.FINGER);
		OPERATIONS.add(DataTypes.BSPSchemaOperations.CAPTURE);
		OPERATIONS.add(DataTypes.BSPSchemaOperations.ENABLE_EVENTS);
		OPERATIONS.add(DataTypes.BSPSchemaOperations.ENROL);
		OPTIONS.add(DataTypes.BSPSchemaOptions.GUI_PROGRESS_EVENTS);
		OPTIONS.add(DataTypes.BSPSchemaOptions.RAW);

		return new BSPSchema(BSP_UUID, BSP_DESCRIPTION, PATH, SPEC_VERSION,
				PRODUCT_VERSION, VENDOR, BSP_SUPPORTED_FORMATS, FACTORS_MASK,
				OPERATIONS, OPTIONS, PAYLOAD_POLICY, MAX_PAYLOAD_SIZE,
				DEFAULT_VERIFY_TIMEOUT, DEFAULT_IDENTIFY_TIMEOUT,
				DEFAULT_CAPTURE_TIMEOUT, DEFAULT_ENROL_TIMEOUT,
				DEFAUT_CALIBRATE_TIMEOUT, MAX_BSP_DB_SIZE, MAX_IDENTIFY,
				MAX_NUM_ENROL_INSTANCES, HOSTING_ENDPOINT_IRI, BSP_ACCESS_UUID,
				BSP_SUPPORTED_ALGORITHMS, BSP_SUPPORTED_TRANSFORM_OPERATIONS);
	}

	@Override
	public BSPSchema getBSPSchema() {
		return bspSchema;
	}

	@Override
	public BIR getBiometricReference() {
		return biometricReference;
	}

	@Override
	public byte[] getACBioInstance() {
		return acBioInstance;
	}

	@Override
	public void bspAttach(String bioAPIVersion,
			ACBioParameters acBioParameters, Vector<UnitListElement> unitList,
			Vector<SecurityProfileType> securityProfileList)
			throws BioAPIException {

		// Check if bsp has been loaded
		if (!bspLoaded) {
			throw new BioAPIException(Facility.BSP,
					BioAPIException.bioAPIErrBSPNotLoaded,
					"You must load the bsp first");
		}

		boolean sensorIncluded = false;
		boolean archiveIncluded = false;
		boolean processingIncluded = false;
		boolean comparisonIncluded = false;

		int unitElementID;
		int currentUnitID;
		boolean unitFound;
		UnitSchema schema;

		for (UnitListElement unitListElement : unitList) {
			unitElementID = unitListElement.getUnitID();
			unitFound = false;
			currentUnitID = 0;
			switch (unitListElement.getUnitCategory()) {
			case ARCHIVE:
				if (!ARCHIVE_SUPPORTED) {
					throw new BioAPIException(Facility.BSP,
							BioAPIException.bioAPIErrInvalidCategory,
							"Archive units are not supported in this BSP");
				}
				if (archiveIncluded) {
					throw new BioAPIException(Facility.BSP,
							BioAPIException.bioAPIBspError,
							"You can attach only one archive unit to the same BSP");
				}
				archiveIncluded = true;
				switch (unitElementID) {
				case DONT_CARE:
					archive = compatibleArchiveUnits.get(0);
					break;
				case DONT_INCLUDE:
					archive = null;
					break;
				default:
					for (int i = 0; i < compatibleArchiveSchemas.size(); i++) {
						schema = compatibleArchiveSchemas.get(i);
						currentUnitID = schema.getUnitID();
						if (unitElementID == currentUnitID) {
							unitFound = true;
							UUID unitManagerUUID = compatibleArchiveSchemas
									.get(i).getUnitManagerUUID();
							if (!unitManagerUUID.equals(BSP_UUID)) {
								// The unit belongs to a BFP
								int realUnitID = unitIDs.get(currentUnitID);
								boolean bfpFound = false;
								for (BFP bfp : loadedBFPs) {
									UUID bfpUUID = bfp.getBFPSchema()
											.getBFPUUID();
									if (bfpUUID.equals(unitManagerUUID)) {
										bfp.unitAttach(realUnitID);
										archive = (Archive) bfp;
										// Set the new BSP UUID for attached
										// unit
										archive.getUnitSchema().setBSPUUID(
												BSP_UUID);
										bfpFound = true;
										break;
									}
								}
								if (!bfpFound) {
									throw new BioAPIException(
											Facility.BSP,
											BioAPIException.bioAPIErrInvalidUUID,
											"The unit's manager UUID doesn't belong to any BFP");
								}
							} else {
								// The unit belongs to this BSP
								archive = compatibleArchiveUnits.get(i);
								archive.getUnitSchema().setBSPUUID(BSP_UUID);
							}
							break;
						}
					}
					if (!unitFound) {
						throw new BioAPIException(Facility.BSP,
								BioAPIException.bioAPIErrInvalidUnitID,
								"Archive unit with id: " + unitElementID
										+ " was not found!");
					}
				}
				break;
			case SENSOR:
				if (!SENSOR_SUPPORTED) {
					throw new BioAPIException(Facility.BSP,
							BioAPIException.bioAPIErrInvalidCategory,
							"Sensor units are not supported in this BSP");
				}

				if (sensorIncluded) {
					throw new BioAPIException(Facility.BSP,
							BioAPIException.bioAPIBspError,
							"You can attach only one sensor unit to the same BSP");
				}
				sensorIncluded = true;
				switch (unitElementID) {
				case DONT_CARE:
					sensor = compatibleSensorUnits.get(0);
					break;
				case DONT_INCLUDE:
					sensor = null;
					break;
				default:
					for (int i = 0; i < compatibleSensorSchemas.size(); i++) {
						int unitID = compatibleSensorSchemas.get(i).getUnitID();
						if (unitElementID == unitID) {
							unitFound = true;
							UUID unitManagerUUID = compatibleSensorSchemas.get(
									i).getUnitManagerUUID();
							if (!unitManagerUUID.equals(BSP_UUID)) {
								int realUnitID = unitIDs.get(unitID);
								boolean bfpFound = false;
								for (BFP bfp : loadedBFPs) {
									UUID bfpUUID = bfp.getBFPSchema()
											.getBFPUUID();
									if (bfpUUID.equals(unitManagerUUID)) {
										bfp.unitAttach(realUnitID);
										sensor = (Sensor) bfp;
										sensor.getUnitSchema().setBSPUUID(
												BSP_UUID);
										bfpFound = true;
										break;
									}
								}
								if (!bfpFound) {
									throw new BioAPIException(
											Facility.BSP,
											BioAPIException.bioAPIErrInvalidUUID,
											"The unit's manager UUID doesn't belong to any BFP");
								}
							} else {
								sensor = compatibleSensorUnits.get(i);
								sensor.getUnitSchema().setBSPUUID(BSP_UUID);
							}

						}
					}
					if (!unitFound) {
						throw new BioAPIException(Facility.BSP,
								BioAPIException.bioAPIErrInvalidUnitID,
								"Sensor unit with id: " + unitElementID
										+ " was not found!");
					}

				}
				break;
			case PROCESSING:
				if (!PROCESSING_SUPPORTED) {
					throw new BioAPIException(Facility.BSP,
							BioAPIException.bioAPIErrInvalidCategory,
							"Processing units are not supported in this BSP");
				}

				if (processingIncluded) {
					throw new BioAPIException(Facility.BSP,
							BioAPIException.bioAPIBspError,
							"You can attach only one processing unit to the same BSP");
				}
				processingIncluded = true;
				switch (unitElementID) {
				case DONT_CARE:
					processing = compatibleProcessingUnits.get(0);
					break;
				case DONT_INCLUDE:
					archive = null;
					break;
				default:
					for (int i = 0; i < compatibleProcessingSchemas.size(); i++) {
						int unitID = compatibleProcessingSchemas.get(i)
								.getUnitID();
						if (unitElementID == unitID) {
							unitFound = true;
							UUID unitManagerUUID = compatibleProcessingSchemas
									.get(i).getUnitManagerUUID();
							if (!unitManagerUUID.equals(BSP_UUID)) {
								int realUnitID = unitIDs.get(unitID);
								boolean bfpFound = false;
								for (BFP bfp : loadedBFPs) {
									UUID bfpUUID = bfp.getBFPSchema()
											.getBFPUUID();
									if (bfpUUID.equals(unitManagerUUID)) {
										bfp.unitAttach(realUnitID);
										processing = (Processing) bfp;
										processing.getUnitSchema().setBSPUUID(
												BSP_UUID);
										bfpFound = true;
										break;
									}
								}
								if (!bfpFound) {
									throw new BioAPIException(
											Facility.BSP,
											BioAPIException.bioAPIErrInvalidUUID,
											"The unit's manager UUID doesn't belong to any BFP");
								}
							} else {
								processing = compatibleProcessingUnits.get(i);
								processing.getUnitSchema().setBSPUUID(BSP_UUID);
							}
						}
					}
					if (!unitFound) {
						throw new BioAPIException(Facility.BSP,
								BioAPIException.bioAPIErrInvalidUnitID,
								"Processing unit with id: " + unitElementID
										+ " was not found!");
					}
				}
				break;
			case COMPARISON:
				if (!COMPARISON_SUPPORTED) {
					throw new BioAPIException(Facility.BSP,
							BioAPIException.bioAPIErrInvalidCategory,
							"Comparison units are not supported in this BSP");
				}

				if (comparisonIncluded) {
					throw new BioAPIException(Facility.BSP,
							BioAPIException.bioAPIBspError,
							"You can attach only one comparison unit to the same BSP");
				}
				comparisonIncluded = true;
				switch (unitElementID) {
				case DONT_CARE:
					comparison = compatibleComparisonUnits.get(0);
					break;
				case DONT_INCLUDE:
					sensor = null;
					break;
				default:
					for (int i = 0; i < compatibleComparisonSchemas.size(); i++) {
						int unitID = compatibleComparisonSchemas.get(i)
								.getUnitID();
						if (unitElementID == unitID) {
							unitFound = true;
							UUID unitManagerUUID = compatibleComparisonSchemas
									.get(i).getUnitManagerUUID();
							if (!unitManagerUUID.equals(BSP_UUID)) {
								int realUnitID = unitIDs.get(unitID);
								boolean bfpFound = false;
								for (BFP bfp : loadedBFPs) {
									UUID bfpUUID = bfp.getBFPSchema()
											.getBFPUUID();
									if (bfpUUID.equals(unitManagerUUID)) {
										bfp.unitAttach(realUnitID);
										comparison = (Comparison) bfp;
										comparison.getUnitSchema().setBSPUUID(
												BSP_UUID);
										bfpFound = true;
										break;
									}
								}
								if (!bfpFound) {
									throw new BioAPIException(
											Facility.BSP,
											BioAPIException.bioAPIErrInvalidUUID,
											"The unit's manager UUID doesn't belong to any BFP");
								}
							} else {
								comparison = compatibleComparisonUnits.get(i);
								comparison.getUnitSchema().setBSPUUID(BSP_UUID);
							}
						}
					}
					if (!unitFound) {
						throw new BioAPIException(Facility.BSP,
								BioAPIException.bioAPIErrInvalidUnitID,
								"Processing unit with id: " + unitElementID
										+ " was not found!");
					}
				}
				break;
			}
		}
	}

	@Override
	public void bspDetach() throws BioAPIException {
		/*
		 * For the moment this function does not need to be implemented because
		 * archive unit do not need to be detached from the system and there are
		 * no BFPs
		 */
	}

	@Override
	public void bspLoad(BFPEventListener bfpEventListener,
			BFPGUIProgressEventListener bfpGUIProgressEventListener,
			BFPEnumerationListener bfpEnumeratorCallback)
			throws BioAPIException {

		UnitSchema unitSchema;

		// Initialize units of this BSP
		SensorUnit sensorUnit = new SensorUnit();
		SQLiteArchiveUnit sqlite = new SQLiteArchiveUnit();
		ProcessingUnit processingUnit = new ProcessingUnit();
		ComparisonUnit comparisonUnit = new ComparisonUnit();

		// Assign IDs to the units of the BSP
		unitSchema = sensorUnit.getUnitSchema();
		unitSchema.setUnitID(++accumulatedUnitID);
		sensorUnit.setUnitSchema(unitSchema);
		compatibleUnits.add(unitSchema);
		compatibleSensorSchemas.add(unitSchema);
		unitIDs.put(accumulatedUnitID, accumulatedUnitID);

		unitSchema = sqlite.getUnitSchema();
		unitSchema.setUnitID(++accumulatedUnitID);
		sqlite.setUnitSchema(unitSchema);
		compatibleUnits.add(unitSchema);
		compatibleArchiveSchemas.add(unitSchema);
		unitIDs.put(accumulatedUnitID, accumulatedUnitID);

		unitSchema = processingUnit.getUnitSchema();
		unitSchema.setUnitID(++accumulatedUnitID);
		processingUnit.setUnitSchema(unitSchema);
		compatibleUnits.add(unitSchema);
		compatibleProcessingSchemas.add(unitSchema);
		unitIDs.put(accumulatedUnitID, accumulatedUnitID);

		unitSchema = comparisonUnit.getUnitSchema();
		unitSchema.setUnitID(++accumulatedUnitID);
		comparisonUnit.setUnitSchema(unitSchema);
		compatibleUnits.add(unitSchema);
		compatibleComparisonSchemas.add(unitSchema);
		unitIDs.put(accumulatedUnitID, accumulatedUnitID);

		// Store BSP unit references locally
		compatibleSensorUnits.add(sensorUnit);
		compatibleArchiveUnits.add(sqlite);
		compatibleProcessingUnits.add(processingUnit);
		compatibleComparisonUnits.add(comparisonUnit);

		/*
		 * Here you may be call bfpEnumeratorCallback function to get all the
		 * compatible BFP references stored in the Component Registry in order
		 * to obtain all compatible units to attach to this BSP
		 */

		bspLoaded = true;
	}

	@Override
	public void bspUnload() throws BioAPIException {
		/*
		 * For the moment is not necessary to implement this function because we
		 * don't have more than one BSP ready to run in the Framework
		 */
	}

	@Override
	public byte checkQuality(BIR inputBIR, DataFormat qualityAlgorithmID)
			throws BioAPIException {
		throw new BioAPIException(Facility.BSP,
				BioAPIException.bioAPIErrFunctionNotSupported,
				UNSUPPORTED_MESSAGE);
	}

	@Override
	public byte[] controlUnit(int unitID, int controlCode, byte[] inputData)
			throws BioAPIException {
		throw new BioAPIException(Facility.BSP,
				BioAPIException.bioAPIErrFunctionNotSupported,
				UNSUPPORTED_MESSAGE);
	}

	@Override
	public UUID enrol(BIR capturedBIR, BIR referenceTemplate,
			Vector<DataTypes.BIRPurpose> purpose,
			DataTypes.BiometricSubtype subtype, DataFormat outputFormat,
			byte[] payload, int timeout, Vector<DataTypes.ResultOptions> options)
			throws BioAPIException {
		throw new BioAPIException(Facility.BSP,
				BioAPIException.bioAPIErrFunctionNotSupported,
				UNSUPPORTED_MESSAGE);
	}

	@Override
	public UUID enrol(Vector<BIR> capturedBIRs, BIR referenceTemplate,
			Vector<DataTypes.BIRPurpose> purpose,
			DataTypes.BiometricSubtype subtype, DataFormat outputFormat,
			byte[] payload, int timeout, Vector<DataTypes.ResultOptions> options)
			throws BioAPIException {
		throw new BioAPIException(Facility.BSP,
				BioAPIException.bioAPIErrFunctionNotSupported,
				UNSUPPORTED_MESSAGE);
	}

	@Override
	public UUID enrol(int numberOfSamples, BIR referenceTemplate,
			Vector<DataTypes.BIRPurpose> purpose,
			DataTypes.BiometricSubtype subtype, DataFormat outputFormat,
			byte[] payload, int timeout, Vector<DataTypes.ResultOptions> options)
			throws BioAPIException {
		throw new BioAPIException(Facility.BSP,
				BioAPIException.bioAPIErrFunctionNotSupported,
				UNSUPPORTED_MESSAGE);
	}

	@Override
	public UUID enrol(BIR capturedBIR, UUID referenceID,
			Vector<DataTypes.BIRPurpose> purpose,
			DataTypes.BiometricSubtype subtype, DataFormat outputFormat,
			byte[] payload, int timeout, Vector<DataTypes.ResultOptions> options)
			throws BioAPIException {
		throw new BioAPIException(Facility.BSP,
				BioAPIException.bioAPIErrFunctionNotSupported,
				UNSUPPORTED_MESSAGE);
	}

	@Override
	public UUID enrol(Vector<BIR> capturedBIRs, UUID referenceID,
			Vector<DataTypes.BIRPurpose> purpose,
			DataTypes.BiometricSubtype subtype, DataFormat outputFormat,
			byte[] payload, int timeout, Vector<DataTypes.ResultOptions> options)
			throws BioAPIException {
		throw new BioAPIException(Facility.BSP,
				BioAPIException.bioAPIErrFunctionNotSupported,
				UNSUPPORTED_MESSAGE);
	}

	@Override
	public UUID enrol(int numberOfSamples, UUID referenceID,
			Vector<DataTypes.BIRPurpose> purpose,
			DataTypes.BiometricSubtype subtype, DataFormat outputFormat,
			byte[] payload, int timeout, Vector<DataTypes.ResultOptions> options)
			throws BioAPIException {
		throw new BioAPIException(Facility.BSP,
				BioAPIException.bioAPIErrFunctionNotSupported,
				UNSUPPORTED_MESSAGE);
	}

	@Override
	public Vector<Candidate> identifyAggregated(int maxFMRrequested,
			DataTypes.BiometricSubtype subtype, boolean binning,
			int maxResults, int timeout, Vector<DataTypes.ResultOptions> options)
			throws BioAPIException {

		BIR bir;
		Vector<Candidate> candidates = null;

		if (progressHandler != null) {
			progressHandler.guiProgressEventCallback(null, 0,
					DataTypes.GUIOperation.IDENTIFY, GUISuboperation.CAPTURE,
					BIRPurpose.IDENTIFY, DataTypes.GUIMoment.BEFORE_START,
					(byte) 0, null, "Fingerprint capture started", null);
		}
		Vector<BIRPurpose> purpose = new Vector<BIRPurpose>();
		purpose.add(BIRPurpose.ENROL_FOR_IDENTIFICATION_ONLY);
		bir = sensor.capture(purpose, subtype, null, timeout, options);
		if (progressHandler != null) {
			progressHandler.guiProgressEventCallback(null, 0,
					DataTypes.GUIOperation.IDENTIFY, GUISuboperation.CAPTURE,
					BIRPurpose.IDENTIFY, DataTypes.GUIMoment.AFTER_END,
					(byte) 0, null, "Fingerprint capture finished", null);
		}

		if (progressHandler != null) {
			progressHandler.guiProgressEventCallback(null, 0,
					DataTypes.GUIOperation.IDENTIFY, GUISuboperation.PROCESS,
					BIRPurpose.IDENTIFY, DataTypes.GUIMoment.BEFORE_START,
					(byte) 0, null, "Fingerprint processing started", null);
		}
		BIR processedBIR;
		processedBIR = processing.process(bir, null);
		if (progressHandler != null) {
			progressHandler.guiProgressEventCallback(null, 0,
					DataTypes.GUIOperation.IDENTIFY, GUISuboperation.PROCESS,
					BIRPurpose.IDENTIFY, DataTypes.GUIMoment.AFTER_END,
					(byte) 0, null, "Fingerprint processing finished", null);
		}

		IdentifyPopulation identifyPopulation;
		identifyPopulation = archive.newIdentifyPopulation();
		comparison.presetIdentifyPopulation(identifyPopulation);

		if (progressHandler != null) {
			progressHandler.guiProgressEventCallback(null, 0,
					DataTypes.GUIOperation.IDENTIFY, GUISuboperation.IDENTIFY,
					BIRPurpose.IDENTIFY, DataTypes.GUIMoment.BEFORE_START,
					(byte) 0, null, "User identification started", null);
		}
		candidates = comparison.identify(maxFMRrequested, processedBIR,
				binning, maxResults, timeout);
		if (progressHandler != null) {
			progressHandler.guiProgressEventCallback(null, 0,
					DataTypes.GUIOperation.IDENTIFY, GUISuboperation.IDENTIFY,
					BIRPurpose.IDENTIFY, DataTypes.GUIMoment.AFTER_END,
					(byte) 0, null, "User identification finished", null);
		}

		return candidates;
	}

	@Override
	public Vector<BFPListElement> queryBFPs(
			Vector<DataTypes.UnitCategoryType> unitCategories)
			throws BioAPIException {
		throw new BioAPIException(Facility.BSP,
				BioAPIException.bioAPIErrFunctionNotSupported,
				UNSUPPORTED_MESSAGE);
	}

	@Override
	public Vector<BFPListElement> queryBFPs() throws BioAPIException {
		throw new BioAPIException(Facility.BSP,
				BioAPIException.bioAPIErrFunctionNotSupported,
				UNSUPPORTED_MESSAGE);
	}

	@Override
	public Vector<UnitSchema> queryUnits(
			Vector<DataTypes.UnitCategoryType> unitCategories)
			throws BioAPIException {
		throw new BioAPIException(Facility.BSP,
				BioAPIException.bioAPIErrFunctionNotSupported,
				UNSUPPORTED_MESSAGE);
	}

	@Override
	public Vector<UnitSchema> queryUnits() throws BioAPIException {
		return compatibleUnits;
	}

	@Override
	public boolean verifyAggregated(int maxFMRrequested, BIR referenceTemplate,
			DataTypes.BiometricSubtype subtype, int timeout,
			Vector<DataTypes.ResultOptions> options) throws BioAPIException {
		throw new BioAPIException(Facility.BSP,
				BioAPIException.bioAPIErrFunctionNotSupported,
				UNSUPPORTED_MESSAGE);
	}

	@Override
	public boolean verifyAggregated(int maxFMRrequested, BIR processedBIR,
			UUID referenceKey, DataTypes.BiometricSubtype subtype, int timeout,
			Vector<DataTypes.ResultOptions> options) throws BioAPIException {
		throw new BioAPIException(Facility.BSP,
				BioAPIException.bioAPIErrFunctionNotSupported,
				UNSUPPORTED_MESSAGE);
	}

	@Override
	public boolean verifyAggregated(int maxFMRrequested, UUID referenceKey,
			DataTypes.BiometricSubtype subtype, int Timeout,
			Vector<DataTypes.ResultOptions> options) throws BioAPIException {
		throw new BioAPIException(Facility.BSP,
				BioAPIException.bioAPIErrFunctionNotSupported,
				UNSUPPORTED_MESSAGE);
	}

	@Override
	public void subscribeToGUIEvents(
			GUISelectEventListener guiSelectEventCallbackFunction,
			GUIStateEventListener guiStateEventCallbackFunction,
			GUIProgressEventListener guiProgressEventCallbackFunction)
			throws BioAPIException {

		if ((guiSelectEventCallbackFunction == null)
				&& (guiStateEventCallbackFunction == null)
				&& (guiProgressEventCallbackFunction == null)) {
			throw new BioAPIException(DataTypes.Facility.BSP,
					BioAPIException.bioAPIErrNoGUIEventHandler,
					"You must provide at least one GUIEventListener");
		}
		if (guiSelectEventCallbackFunction != null) {
			selectHandler = guiSelectEventCallbackFunction;
		}
		if (guiStateEventCallbackFunction != null) {
			stateHandler = guiStateEventCallbackFunction;
		}
		if (guiProgressEventCallbackFunction != null) {
			progressHandler = guiProgressEventCallbackFunction;
		}
	}

	@Override
	public void unsubscribeFromGUIEvents() throws BioAPIException {
		selectHandler = null;
		stateHandler = null;
		progressHandler = null;
	}

	public BIR capture(Vector<DataTypes.BIRPurpose> purpose,
			BiometricSubtype subtype, DataFormat outputFormat, int timeout,
			Vector<DataTypes.ResultOptions> options) throws BioAPIException {
		biometricReference = sensor.capture(purpose, subtype, outputFormat,
				timeout, options);
		return biometricReference;
	}

	@Override
	public BIR createTemplate(BIR capturedBIR, BIR referenceTemplate,
			DataFormat outputFormat, byte[] payload) throws BioAPIException {
		return processing.createTemplate(capturedBIR, referenceTemplate,
				outputFormat, payload);
	}

	@Override
	public BIR createTemplate(Vector<BIR> capturedBIRs, BIR referenceTemplate,
			DataFormat outputFormat, byte[] payload) throws BioAPIException {
		return processing.createTemplate(capturedBIRs, referenceTemplate,
				outputFormat, payload);
	}

	@Override
	public BIR process(BIR capturedBIR, DataFormat outputFormat)
			throws BioAPIException {
		return processing.process(capturedBIR, outputFormat);
	}

	@Override
	public BIR process(BIR capturedBIR, Vector<BIR> auxiliaryBIRs,
			DataFormat outputFormat) throws BioAPIException {
		return processing.process(capturedBIR, outputFormat);
	}

	@Override
	public int getFMRAchieved() {
		return comparison.getFMRAchieved();
	}

	@Override
	public BIR getAdaptedBIR() {
		return comparison.getAdaptedBIR();
	}

	@Override
	public byte[] getPayload() {
		return comparison.getPayload();
	}

	@Override
	public Vector<Candidate> identify(int maxFMRrequested, BIR processedBIR,
			boolean binning, int maxResults, int timeout)
			throws BioAPIException {
		return comparison.identify(maxFMRrequested, processedBIR, binning,
				maxResults, timeout);
	}

	@Override
	public Vector<Candidate> identify(int maxFMRrequested, BIR processedBIR,
			Vector<BIR> auxiliaryBIRs, boolean binning, int maxResults,
			int timeout) throws BioAPIException {
		return comparison.identify(maxFMRrequested, processedBIR,
				auxiliaryBIRs, binning, maxResults, timeout);
	}

	@Override
	public void presetIdentifyPopulation(IdentifyPopulation population)
			throws BioAPIException {
		comparison.presetIdentifyPopulation(population);
	}

	@Override
	public boolean verify(int maxFMRrequested, BIR processedBIR,
			BIR referenceTemplate, Vector<ResultOptions> options)
			throws BioAPIException {
		return comparison.verify(maxFMRrequested, processedBIR,
				referenceTemplate, options);
	}

	@Override
	public boolean verify(int maxFMRrequested, BIR processedBIR,
			BIR referenceTemplate, Vector<BIR> auxiliaryBIRs,
			Vector<ResultOptions> options) throws BioAPIException {
		return comparison.verify(maxFMRrequested, processedBIR,
				referenceTemplate, auxiliaryBIRs, options);
	}

	@Override
	public void closeDatabase() throws BioAPIException {
		archive.closeDatabase();
	}

	@Override
	public void deleteBIR(UUID key) throws BioAPIException {
		archive.deleteBIR(key);
	}

	@Override
	public BIR getSingleBIR(UUID key) throws BioAPIException {
		return archive.getSingleBIR(key);
	}

	@Override
	public Vector<UUID> listUUIDs() throws BioAPIException {
		return archive.listUUIDs();
	}

	@Override
	public IdentifyPopulation newIdentifyPopulation() throws BioAPIException {
		return archive.newIdentifyPopulation();
	}

	@Override
	public IdentifyPopulation newIdentifyPopulation(Vector<UUID> uuidList)
			throws BioAPIException {
		return archive.newIdentifyPopulation(uuidList);
	}

	@Override
	public IdentifyPopulation newIdentifyPopulation(byte[] query)
			throws BioAPIException {
		return archive.newIdentifyPopulation(query);
	}

	@Override
	public void openDatabase(byte[] databaseID, BIRDatabaseAccess access)
			throws BioAPIException {
		archive.openDatabase(databaseID, access);
	}

	@Override
	public UUID storeBIR(BIR biometricReference) throws BioAPIException {
		return archive.storeBIR(biometricReference);
	}

	@Override
	public void storeBIR(BIR biometricReference, UUID key)
			throws BioAPIException {
		archive.storeBIR(biometricReference, key);
	}

	@Override
	public UUID storeBIR(BIR biometricReference, byte[] auxiliaryData)
			throws BioAPIException {
		return archive.storeBIR(biometricReference, auxiliaryData);
	}

	@Override
	public void storeBIR(BIR biometricReference, byte[] auxiliaryData, UUID key)
			throws BioAPIException {
		archive.storeBIR(biometricReference, auxiliaryData, key);
	}

	@Override
	public UnitSchema getUnitSchema() {
		/* This function are under revision */
		return null;
	}

	@Override
	public void setUnitSchema(UnitSchema unitSchema) {
		/* This function are under revision */
	}

	@Override
	public byte[] getAuxiliaryData() {
		return sensor.getAuxiliaryData();
	}

	@Override
	public UnitIndicatorStatus getIndicatorStatus() {
		return sensor.getIndicatorStatus();
	}

	@Override
	public void calibrate(int timeout) throws BioAPIException {
		sensor.calibrate(timeout);
	}

	@Override
	public byte[] controlUnit(int controlCode, byte[] inputData)
			throws BioAPIException {
		return sensor.controlUnit(controlCode, inputData);
	}

	@Override
	public void setIndicatorStatus(UnitIndicatorStatus indicatorStatus)
			throws BioAPIException {
		sensor.setIndicatorStatus(indicatorStatus);
	}

	@Override
	public void setPowerMode(UnitPowerMode powerMode) throws BioAPIException {
		sensor.setPowerMode(powerMode);
	}

	/* %%%%%%%%%% UNITS %%%%%%%%%% */

	/* CAPTURE UNIT */
	private class SensorUnit implements Sensor {

		@Override
		public UnitSchema getUnitSchema() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public void setUnitSchema(UnitSchema unitSchema) {
			// TODO Auto-generated method stub
		}

		@Override
		public byte[] getACBioInstance() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public byte[] getAuxiliaryData() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public UnitIndicatorStatus getIndicatorStatus() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public void calibrate(int timeout) throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public BIR capture(Vector<BIRPurpose> purpose,
				BiometricSubtype subtype, DataFormat outputFormat, int timeout,
				Vector<ResultOptions> options) throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public byte[] controlUnit(int controlCode, byte[] inputData)
				throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public void setIndicatorStatus(UnitIndicatorStatus indicatorStatus)
				throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public void setPowerMode(UnitPowerMode powerMode)
				throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}
	}

	/* ARCHIVE UNIT */
	private class SQLiteArchiveUnit implements Archive {

		private final DataTypes.UnitCategoryType UNIT_CATEGORY = DataTypes.UnitCategoryType.ARCHIVE;
		private final UUID UNIT_PROPERTIES = null;
		private final String VENDOR_INFORMATION = "SQLite-ArchiveUnit";
		private final Vector<DataTypes.EventKind> SUPPORTED_EVENTS = null;
		private final UUID UNIT_PROPERTY_ID = null;
		private final byte[] UNIT_PROPERTY = null;
		private final String HARDWARE_VERSION = "demo1.0";
		private final String FIRMWARE_VERSION = "demo1.0";
		private final String SOFTWARE_VERSION = "demo1.0";
		private final String HARDWARE_SERIAL_NUMBER = "demo1.0";
		private final boolean AUTHENTICATED_HARDWARE = false;
		private final int MAX_BSP_DB_SIZE_UNIT = 0;
		private final int MAX_IDENTIFY_UNIT = 0xFFFFFFFF;
		private final Vector<SecurityProfileType> SECURITY_PROFILE = null;

		private UnitSchema unitSchema;
		private ACBioParameters acBio;

		private SQLiteArchiveUnitDatabase databaseHelper;
		private SQLiteDatabase sqlDatabase;

		public SQLiteArchiveUnit() {
			this.unitSchema = new UnitSchema(null, BSP_UUID, 2, UNIT_CATEGORY,
					UNIT_PROPERTIES, VENDOR_INFORMATION, SUPPORTED_EVENTS,
					UNIT_PROPERTY_ID, UNIT_PROPERTY, HARDWARE_VERSION,
					FIRMWARE_VERSION, SOFTWARE_VERSION, HARDWARE_SERIAL_NUMBER,
					AUTHENTICATED_HARDWARE, MAX_BSP_DB_SIZE_UNIT,
					MAX_IDENTIFY_UNIT, SECURITY_PROFILE);

			this.acBio = null;
		}

		@Override
		public UnitSchema getUnitSchema() {
			return unitSchema;
		}

		@Override
		public void setUnitSchema(UnitSchema unitSchema) {
			this.unitSchema = unitSchema;
		}

		@Override
		public byte[] getACBioInstance() {
			return null;
		}

		@Override
		public void closeDatabase() throws BioAPIException {
			sqlDatabase.close();
			sqlDatabase = null;
			databaseHelper.close();
			databaseHelper = null;
		}

		@Override
		public byte[] controlUnit(int controlCode, byte[] inputData)
				throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public void deleteBIR(UUID key) throws BioAPIException {
			if (sqlDatabase == null && !sqlDatabase.isOpen()) {
				throw new BioAPIException(Facility.UNIT,
						BioAPIException.bioAPIErrDatabaseDoesNotExist,
						"Database is not open"/* TODO Resource */);
			} else {
				if (sqlDatabase.delete("Users", "uuid=" + key.toString(), null) == 0) {
					throw new BioAPIException(Facility.UNIT,
							BioAPIException.bioAPIErrRecordNotFound,
							"Record not found in the database"/* TODO Resource */);
				}
			}
		}

		@Override
		public BIR getSingleBIR(UUID key) throws BioAPIException {
			if (sqlDatabase == null || !sqlDatabase.isOpen()) {
				throw new BioAPIException(Facility.UNIT,
						BioAPIException.bioAPIErrDatabaseDoesNotExist,
						"Database is not open");
			} else {
				Cursor cursor = sqlDatabase.rawQuery(
						"SELECT biometricData FROM Users WHERE uuid=\""
								+ key.toString() + "\";", null);
				if (cursor.moveToFirst()) {
					BIR biometricReference = new BIR();
					biometricReference.birFromByteArray(cursor.getBlob(0));
					return biometricReference;
				} else {
					throw new BioAPIException(Facility.UNIT,
							BioAPIException.bioAPIErrRecordNotFound,
							"Record not found in the database"/* TODO Resource */);
				}
			}
		}

		@Override
		public Vector<UUID> listUUIDs() throws BioAPIException {
			if (sqlDatabase == null || !sqlDatabase.isOpen()) {
				throw new BioAPIException(Facility.UNIT,
						BioAPIException.bioAPIErrDatabaseDoesNotExist,
						"Database is not open"/* TODO Resource */);
			} else {
				Cursor cursor = sqlDatabase.rawQuery("SELECT uuid FROM Users;",
						null);
				cursor.moveToFirst();

				int numberOfUsers = cursor.getCount();
				Vector<UUID> uuidList = new Vector<UUID>();
				for (int i = 0; i < numberOfUsers; i++) {
					uuidList.add(UUID.fromString(cursor.getString(0)));
					cursor.moveToNext();
				}
				return uuidList;
			}
		}

		@Override
		public IdentifyPopulation newIdentifyPopulation()
				throws BioAPIException {
			return newIdentifyPopulation(listUUIDs());
		}

		@Override
		public IdentifyPopulation newIdentifyPopulation(Vector<UUID> uuidList)
				throws BioAPIException {
			if (sqlDatabase == null || !sqlDatabase.isOpen()) {
				throw new BioAPIException(Facility.UNIT,
						BioAPIException.bioAPIErrDatabaseDoesNotExist,
						"Database is not open"/* TODO Resource */);
			} else {
				Vector<PopulationMember> members = new Vector<PopulationMember>();
				PopulationMember member = null;
				BIR biometricReference = null;
				UUID currentUUID = null;
				for (int i = 0; i < uuidList.size(); i++) {
					currentUUID = uuidList.get(i);
					biometricReference = getSingleBIR(currentUUID);
					member = new PopulationMember(currentUUID,
							biometricReference);
					members.add(member);
				}
				IdentifyPopulation population = new IdentifyPopulation(members);
				return population;
			}
		}

		@Override
		public IdentifyPopulation newIdentifyPopulation(byte[] query)
				throws BioAPIException {
			throw new BioAPIException(Facility.BSP,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public void openDatabase(byte[] databaseID, BIRDatabaseAccess access)
				throws BioAPIException {
			String databaseName = new String(databaseID);
			databaseHelper = new SQLiteArchiveUnitDatabase(applicationContext,
					databaseName, null, 1);
			if (access == BIRDatabaseAccess.READ) {
				sqlDatabase = databaseHelper.getReadableDatabase();
			}
			if (access == BIRDatabaseAccess.WRITE
					|| access == BIRDatabaseAccess.READ_WRITE) {
				sqlDatabase = databaseHelper.getWritableDatabase();
			}
		}

		@Override
		public UUID storeBIR(BIR biometricReference) throws BioAPIException {
			byte[] auxiliaryData = null;
			return storeBIR(biometricReference, auxiliaryData);
		}

		@Override
		public void storeBIR(BIR biometricReference, UUID key)
				throws BioAPIException {
			storeBIR(biometricReference, null, key);
		}

		@Override
		public UUID storeBIR(BIR biometricReference, byte[] auxiliaryData)
				throws BioAPIException {

			if (progressHandler != null) {
				progressHandler.guiProgressEventCallback(null, 0,
						DataTypes.GUIOperation.ENROL,
						GUISuboperation.CREATE_TEMPLATE, BIRPurpose.ENROL,
						DataTypes.GUIMoment.BEFORE_START, (byte) 0, null,
						"Storing fingerprint", null);
			}

			UUID uuid;
			if (sqlDatabase == null && !sqlDatabase.isOpen()) {
				throw new BioAPIException(Facility.UNIT,
						BioAPIException.bioAPIErrDatabaseDoesNotExist,
						"Database is not open"/* TODO Resource */);
			} else {
				if (sqlDatabase.isReadOnly()) {
					throw new BioAPIException(Facility.UNIT,
							BioAPIException.bioAPIErrInvalidAccessRequest,
							"Database was opened for read-only operations"/*
																		 * TODO
																		 * Resource
																		 */);
				} else {
					uuid = UUID.randomUUID();
					while (findUUID(uuid)) {
						uuid = UUID.randomUUID();
					}

					storeData(uuid, biometricReference, auxiliaryData);

				}
			}

			if (progressHandler != null) {
				progressHandler.guiProgressEventCallback(null, 0,
						DataTypes.GUIOperation.ENROL,
						GUISuboperation.CREATE_TEMPLATE, BIRPurpose.ENROL,
						DataTypes.GUIMoment.AFTER_END, (byte) 0, null, null,
						null);
			}

			return uuid;

		}

		@Override
		public void storeBIR(BIR biometricReference, byte[] auxiliaryData,
				UUID key) throws BioAPIException {
			if (progressHandler != null) {
				progressHandler.guiProgressEventCallback(null, 0,
						DataTypes.GUIOperation.ENROL,
						GUISuboperation.CREATE_TEMPLATE, BIRPurpose.ENROL,
						DataTypes.GUIMoment.BEFORE_START, (byte) 0, null,
						"Storing fingerprint", null);
			}

			if (sqlDatabase == null && !sqlDatabase.isOpen()) {
				throw new BioAPIException(Facility.UNIT,
						BioAPIException.bioAPIErrDatabaseDoesNotExist,
						"Database is not open");
			} else {
				if (sqlDatabase.isReadOnly()) {
					throw new BioAPIException(Facility.UNIT,
							BioAPIException.bioAPIErrInvalidAccessRequest,
							"Database was opened for read-only operations");
				} else {
					if (findUUID(key)) {
						throw new BioAPIException(Facility.UNIT, 0xFF,
								"The UUID provided already exists");
					} else {
						storeData(key, biometricReference, auxiliaryData);
					}
				}
			}
			if (progressHandler != null) {
				progressHandler.guiProgressEventCallback(null, 0,
						DataTypes.GUIOperation.ENROL,
						GUISuboperation.CREATE_TEMPLATE, BIRPurpose.ENROL,
						DataTypes.GUIMoment.AFTER_END, (byte) 0, null, null,
						null);
			}
		}

		private boolean findUUID(UUID uuid) throws BioAPIException {
			Vector<UUID> uuids = listUUIDs();
			if (uuids.contains(uuid))
				return true;
			return false;
		}

		private void storeData(UUID uuid, BIR bir, byte[] auxiliaryData)
				throws BioAPIException {
			ContentValues cv = new ContentValues();
			cv.put("uuid", uuid.toString());
			cv.put("biometricData", bir.birToByteArray());
			if (auxiliaryData != null)
				cv.put("auxiliaryData", auxiliaryData);
			sqlDatabase.insert("Users", null, cv);
		}

		public class SQLiteArchiveUnitDatabase extends SQLiteOpenHelper {

			private String sqlCreate;

			public SQLiteArchiveUnitDatabase(Context context,
					String databaseName, CursorFactory factory, int version) {
				super(context, databaseName, factory, version);
				this.sqlCreate = "CREATE TABLE Users (uuid TEXT, biometricData BLOB, auxiliaryData BLOB)";
			}

			@Override
			public void onCreate(SQLiteDatabase db) {
				db.execSQL(sqlCreate);
			}

			@Override
			public void onUpgrade(SQLiteDatabase db, int oldVersion,
					int newVersion) {
			}

		}
	}

	/* PROCESSING UNIT */
	private class ProcessingUnit implements Processing {

		@Override
		public UnitSchema getUnitSchema() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public void setUnitSchema(UnitSchema unitSchema) {
			// TODO Auto-generated method stub

		}

		@Override
		public byte[] getACBioInstance() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public byte[] controlUnit(int controlCode, byte[] inputData)
				throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public BIR createTemplate(BIR capturedBIR, BIR referenceTemplate,
				DataFormat outputFormat, byte[] payload) throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public BIR createTemplate(Vector<BIR> capturedBIRs,
				BIR referenceTemplate, DataFormat outputFormat, byte[] payload)
				throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public BIR process(BIR capturedBIR, DataFormat outputFormat)
				throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public BIR process(BIR capturedBIR, Vector<BIR> auxiliaryBIRs,
				DataFormat outputFormat) throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

	}

	/* COMPARISON UNIT */
	private class ComparisonUnit implements Comparison {

		@Override
		public UnitSchema getUnitSchema() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public void setUnitSchema(UnitSchema unitSchema) {
			// TODO Auto-generated method stub

		}

		@Override
		public byte[] getACBioInstance() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public int getFMRAchieved() {
			// TODO Auto-generated method stub
			return 0;
		}

		@Override
		public BIR getAdaptedBIR() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public byte[] getPayload() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public byte[] controlUnit(int controlCode, byte[] inputData)
				throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public Vector<Candidate> identify(int maxFMRrequested,
				BIR processedBIR, boolean binning, int maxResults, int timeout)
				throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public Vector<Candidate> identify(int maxFMRrequested,
				BIR processedBIR, Vector<BIR> auxiliaryBIRs, boolean binning,
				int maxResults, int timeOut) throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public void presetIdentifyPopulation(IdentifyPopulation population)
				throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public boolean verify(int maxFMRrequested, BIR processedBIR,
				BIR referenceTemplate, Vector<ResultOptions> options)
				throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}

		@Override
		public boolean verify(int maxFMRrequested, BIR processedBIR,
				BIR referenceTemplate, Vector<BIR> auxiliaryBIRs,
				Vector<ResultOptions> options) throws BioAPIException {
			throw new BioAPIException(Facility.UNIT,
					BioAPIException.bioAPIErrFunctionNotSupported,
					UNSUPPORTED_MESSAGE);
		}
	}

}
