package org.bioapi;

import java.util.UUID;
import java.util.Vector;
import org.bioapi.data.BIR;
import org.bioapi.data.DataTypes.BIRDatabaseAccess;
import org.bioapi.data.IdentifyPopulation;
import org.bioapi.data.UnitSchema;

/**
 * Interface for attached Archive units.
 * 
 * @author See CREDITS.txt
 */
public interface Archive {

	UnitSchema getUnitSchema();

	void setUnitSchema(UnitSchema unitSchema);

	/**
	 * If ACBio is supported by this Unit, then the Unit shall update this
	 * property with the last generated ACBio instance. If ACBio is not
	 * supported, this property shall be fixed to NULL.
	 */
	byte[] getACBioInstance();

	/**
	 * Closes the access to the database for which the Unit is developed
	 * 
	 * @throws BioAPIException
	 *             If operation fails
	 */
	void closeDatabase() throws BioAPIException;

	/**
	 * Sends control data to the BioAPI_Unit and receives status or operation
	 * data from there. The content of the parameters and the output will be
	 * specified in the related interface specification of this BioAPI_Unit
	 * 
	 * @param controlCode
	 *            The function code in the BioAPI_Unit to be called
	 * @param inputData
	 *            Buffer containing the data to be sent to the BioAPI unit
	 *            related to the given ControlCode
	 * @return Data buffer containing the data received from the BioAPI_Unit
	 *         after processing the function indicated by the ControlCode. If no
	 *         memory block has been allocated by the function the value shall
	 *         be NULL.
	 * @throws BioAPIException
	 *             If case of any error
	 */
	byte[] controlUnit(int controlCode, byte[] inputData)
			throws BioAPIException;

	/**
	 * Deletes the BIR from database
	 * 
	 * @param key
	 *            UUID of the record to be deleted
	 * @throws BioAPIException
	 *             If the database has been closed, or if database was opened in
	 *             read-only mode, or any other kind of error
	 */
	void deleteBIR(UUID key) throws BioAPIException;

	/**
	 * Get the specified record
	 * 
	 * @param key
	 *            UUID of the record to retrieve
	 * @return The requested record
	 * @throws BioAPIException
	 *             If the database has been closed, or the database was opened
	 *             in write-only mode, or the record is not found, or any other
	 *             kind of error
	 */
	BIR getSingleBIR(UUID key) throws BioAPIException;

	/**
	 * Returns the list of UUIDs included into the opened database
	 * 
	 * @return The list of UUIDs included into the opened database
	 * @throws BioAPIException
	 *             If the database has been closed, or the database was opened
	 *             in write-only mode, or any other kind of error
	 */
	Vector<UUID> listUUIDs() throws BioAPIException;

	/**
	 * The database attached to the BSP or BFP is set to be the data source for
	 * an identification operation. This can also be used to get a list of the
	 * BIRs included in the database.
	 * 
	 * @return The object representing the population (collection of BIRs)
	 * @throws BioAPIException
	 *             If the database has been closed, or the database was opened
	 *             in write-only mode, or any other kind of error
	 */
	IdentifyPopulation newIdentifyPopulation() throws BioAPIException;

	/**
	 * The database attached to the BSP or BFP is set to be the data source for
	 * an identification operation. This can also be used to get a list of the
	 * BIRs included in the database.
	 * 
	 * @param uuidList
	 *            The list of UUIDs to be included into the new identify
	 *            population
	 * @return The object representing the population (collection of BIRs)
	 * @throws BioAPIException
	 *             If the database has been closed, or the database was opened
	 *             in write-only mode, or any other kind of error
	 */
	IdentifyPopulation newIdentifyPopulation(Vector<UUID> uuidList)
			throws BioAPIException;

	/**
	 * The database attached to the BSP or BFP is set to be the data source for
	 * an identification operation. This can also be called to obtain the list
	 * of BIRs that match an specified criteria in the database.
	 * 
	 * @param query
	 *            The query submitted to the database to select the users to be
	 *            included into the population from which to identify. The query
	 *            is submitted as a byte array, but following the specific
	 *            format demanded by the database engine used.
	 * @return The object representing the population (collection of BIRs)
	 * @throws BioAPIException
	 *             If the database has been closed, or the database was opened
	 *             in write-only mode, or no records have been found, or any
	 *             other kind of error
	 */
	IdentifyPopulation newIdentifyPopulation(byte[] query)
			throws BioAPIException;

	/**
	 * Opens the database managed by the BSP or BFP. No other database shall be
	 * opened at the same time within the same unit. If a different database has
	 * been previously used, then the CloseDatabase method shall be called
	 * before calling OpenDatabase.
	 * 
	 * @param databaseID
	 *            specifies an optional identifier for the database to be
	 *            opened. When different databases can be used with a single
	 *            Archive unit, all databases shall be compatible with the
	 *            internal procedures of the Archive unit. If the unit does not
	 *            allow the selection of different databases, then this
	 *            parameter shall have a NULL value.
	 * @param access
	 *            specifies the access mode of the database to be opened
	 *            (read/write).
	 * @throws BioAPIException
	 */
	void openDatabase(byte[] databaseID, BIRDatabaseAccess access)
			throws BioAPIException;

	/**
	 * Add the specified BIR to database with no UUID, allowing the Unit to
	 * return the UUID assigned
	 * 
	 * @param biometricReference
	 *            The specific BIR to store
	 * @return The ID of the newly added BIR
	 * @throws BioAPIException
	 *             If the database has been closed, or the database was opened
	 *             in read-only mode, or any other kind of error
	 */
	UUID storeBIR(BIR biometricReference) throws BioAPIException;

	/**
	 * Add the specified BIR to database and assign it the UUID provided. If the
	 * UUID is already assigned, throw an exception. If the program wanted to
	 * update an existing UUID, the application should first delete it and then
	 * re-use the UUID. This is done this way to avoid involuntary overwriting.
	 * 
	 * @param biometricReference
	 *            The specific BIR to store
	 * @param key
	 *            The assigned UUID to that BIR
	 * @throws BioAPIException
	 *             If the database has been closed, or the database was opened
	 *             in read-only mode, or the UUID is already in use, or any
	 *             other kind of error
	 */
	void storeBIR(BIR biometricReference, UUID key) throws BioAPIException;

	/**
	 * Add the specified BIR to database with no UUID, allowing the Unit to
	 * return the UUID assigned. A set of bytes with additional information is
	 * submitted in AuxiliaryData. The format of that data is to be understood
	 * by the Unit (not specified in this International Standard). This
	 * information can be, for example, the demographics associated to the
	 * BiometricReference.
	 * 
	 * @param biometricReference
	 *            The specific BIR to store
	 * @param auxiliaryData
	 *            Additional data to be stored into the database in reference to
	 *            the BIR stored.
	 * @return The ID of the newly added BIR
	 * @throws BioAPIException
	 *             If the database has been closed, or the database was opened
	 *             in read-only mode, or any other kind of error
	 */
	UUID storeBIR(BIR biometricReference, byte[] auxiliaryData)
			throws BioAPIException;

	/**
	 * Add the specified BIR to database and assign it the UUID provided. If the
	 * UUID is already assigned, throw an exception. If the program wanted to
	 * update an existing UUID, the application should first delete it and then
	 * re-use the UUID. This is done this way to avoid involuntary overwriting.
	 * A set of bytes with additional information is submitted in AuxiliaryData.
	 * The format of that data is to be understood by the Unit (not specified in
	 * this International Standard). This information can be, for example, the
	 * demographics associated to the BiometricReference.
	 * 
	 * @param biometricReference
	 *            The specific BIR to store
	 * @param auxiliaryData
	 *            Additional data to be stored into the database in reference to
	 *            the BIR stored.
	 * @param key
	 *            The assigned UUID to that BIR
	 * @throws BioAPIException
	 *             If the database has been closed, or the database was opened
	 *             in read-only mode, or the UUID is already in use, or any
	 *             other kind of error
	 */
	void storeBIR(BIR biometricReference, byte[] auxiliaryData, UUID key)
			throws BioAPIException;

}
