package org.bioapi;

import java.util.UUID;
import java.util.Vector;
import org.bioapi.data.BFPListElement;
import org.bioapi.data.BIR;
import org.bioapi.data.BSPSchema;
import org.bioapi.data.Candidate;
import org.bioapi.data.DataFormat;
import org.bioapi.data.DataTypes;
import org.bioapi.data.UnitSchema;

/**
 * This interface represents the attached BSP session. Through this interface,
 * all methods and properties from the BSP are accessible to the application
 * through the Framework.
 */
public interface AttachSession extends Sensor, Archive, Processing, Comparison {

	BSPSchema getBSPSchema();

	BIR getBiometricReference();

	byte[] getACBioInstance();

	/**
	 * This function performs a quality assessment of the biometric data
	 * contained within an input BIR. <br/>
	 * If a quality algorithm is specified, and that algorithm is supported by
	 * the BSP, it shall be utilized. If NULL, the BSP will select the quality
	 * algorithm to apply. BSPs may determine what quality algorithms are
	 * supported by calling BioAPI_EnumBSPs. If an unsupported algorithm is
	 * requested, a BioAPIERR_UNSUPPORTED_ALGORITHM BioAPIException shall be
	 * thrown.
	 * 
	 * @param inputBIR
	 *            The BIR containing the biometric data whose quality is to be
	 *            assessed.
	 * @param qualityAlgorithmID
	 *            As input, the quality algorithm to be used by the BSP; as
	 *            output, the quality algorithm actually used by the BSP.
	 * @return The assessed quality of the BIR data.
	 * @throws BioAPIException
	 *             If case of any error
	 */
	byte checkQuality(BIR inputBIR, DataFormat qualityAlgorithmID)
			throws BioAPIException;

	/**
	 * Sends control data to the BioAPI_Unit and receives status or operation
	 * data from there. The content of the parameters and the output will be
	 * specified in the related interface specification of this BioAPI_Unit
	 * 
	 * @param unitID
	 *            identifier of the unit to which the ControlUnit function is
	 *            forwarded
	 * @param controlCode
	 *            The function code in the BioAPI_Unit to be called
	 * @param inputData
	 *            Buffer containing the data to be sent to the BioAPI unit
	 *            related to the given ControlCode
	 * @return Data buffer containing the data received from the BioAPI_Unit
	 *         after processing the function indicated by the ControlCode. If no
	 *         memory block has been allocated by the function the value shall
	 *         be NULL.
	 * @throws BioAPIException
	 *             If case of any error
	 */
	byte[] controlUnit(int unitID, int controlCode, byte[] inputData)
			throws BioAPIException;

	/**
	 * Enrol a user by means of (in order of appereance in the list above): <br/>
	 * <ul>
	 * <li>Providing the sample to use for the enrolment in the first parameter
	 * of the method</li>
	 * <li>Providing a list of samples to use for the enrolment in the first
	 * parameter of the method</li>
	 * <li>Requiring the capture process of a certain number of times to the
	 * Sensor unit in the BSP</li>
	 * </ul>
	 * <br/>
	 * The enrolment can be done from scratch, or by updating a previous
	 * enrolment. In the first case, the parameter ReferenceTemplate or
	 * ReferenceID shall have a Null value. In the second case, the biometric
	 * reference to be updated can be provided by means of a BIR (using
	 * ReferenceTemplate) or by means of the UUID (using ReferenceID). <br/>
	 * <li>The result of a successful enrolment will be the UUID assigned to the
	 * biometric reference created and, optionally, the BIR for that biometric
	 * reference, which will be located in the BiometricReference property. If
	 * for any reason (e.g. security concenrs) the BSP does not want to publish
	 * the generated biometric reference, that property shall be set to NULL.</li>
	 * <br/>
	 * The BSP is responsible for providing the user interface associated with
	 * the enrolment operation as a default. The application may request control
	 * of the GUI "look-and-feel" by providing a GUI callback via the
	 * BSP.subscribeToGUIEvents() method. Since the enrol() operation includes
	 * capture, it serializes the use of the sensor device. If two or more
	 * applications are racing for the device, the losers will wait until the
	 * operation completes or the timeout expires. This serialization takes
	 * place in all functions that capture data. The BSP is responsible for
	 * serializing. It may do this by either throwing an exception to indicate
	 * that the device is busy or by queuing requests.
	 * 
	 * @param capturedBIR
	 *            Optionally, the biometric sample to enrol, in BIR format
	 * @param referenceTemplate
	 *            Optionally, the BIR of the biometric reference to be updated
	 * @param purpose
	 *            A value indicating the desired purpose (Enrol,
	 *            EnrolForVerificationOnly, or EnrolForIdentification only).
	 * @param subtype
	 *            Specifies which subtype (e.g., left/right eye) to enrol. A
	 *            'null' value indicates that the BSP is to select the
	 *            subtype(s)
	 * @param outputFormat
	 *            Specifies which BDB format to use for the returned
	 *            NewTemplate, if the BSP supports more than one format. A NULL
	 *            value indicates that the BSP is to select the format
	 * @param payload
	 *            A payload that will be stored by the BSP.
	 * @param timeout
	 *            An integer specifying the timeout value (in milliseconds) for
	 *            the operation. If this timeout is reached, the function
	 *            returns an error, and no results. This value can be any
	 *            positive number. A ‘-1’ value means the BSP’s default
	 *            timeout value will be used.
	 * @param options
	 *            Requests additional output such as audit data. This data may
	 *            be used to provide a human-identifiable data of the person at
	 *            the sensor unit
	 * @return The UUID for the biometric reference stored
	 * @throws BioAPIException
	 *             If any of the parameters is not correct or other error occur
	 *             during the process (e.g. no access to the database)
	 */
	UUID enrol(BIR capturedBIR, BIR referenceTemplate,
			Vector<DataTypes.BIRPurpose> purpose,
			DataTypes.BiometricSubtype subtype, DataFormat outputFormat,
			byte[] payload, int timeout, Vector<DataTypes.ResultOptions> options)
			throws BioAPIException;

	UUID enrol(Vector<BIR> capturedBIRs, BIR referenceTemplate,
			Vector<DataTypes.BIRPurpose> purpose,
			DataTypes.BiometricSubtype subtype, DataFormat outputFormat,
			byte[] payload, int timeout, Vector<DataTypes.ResultOptions> options)
			throws BioAPIException;

	UUID enrol(int numberOfSamples, BIR referenceTemplate,
			Vector<DataTypes.BIRPurpose> purpose,
			DataTypes.BiometricSubtype subtype, DataFormat outputFormat,
			byte[] payload, int timeout, Vector<DataTypes.ResultOptions> options)
			throws BioAPIException;

	UUID enrol(BIR capturedBIR, UUID referenceID,
			Vector<DataTypes.BIRPurpose> purpose,
			DataTypes.BiometricSubtype subtype, DataFormat outputFormat,
			byte[] payload, int timeout, Vector<DataTypes.ResultOptions> options)
			throws BioAPIException;

	UUID enrol(Vector<BIR> capturedBIRs, UUID referenceID,
			Vector<DataTypes.BIRPurpose> purpose,
			DataTypes.BiometricSubtype subtype, DataFormat outputFormat,
			byte[] payload, int timeout, Vector<DataTypes.ResultOptions> options)
			throws BioAPIException;

	UUID enrol(int numberOfSamples, UUID referenceID,
			Vector<DataTypes.BIRPurpose> purpose,
			DataTypes.BiometricSubtype subtype, DataFormat outputFormat,
			byte[] payload, int timeout, Vector<DataTypes.ResultOptions> options)
			throws BioAPIException;

	/**
	 * This method provides an aggregated functionality. It increases the
	 * functionlality of the Identify method from Comparison (see clause 4.2),
	 * allowing the Identify operation when the biometric sample is directly
	 * captured by the BSP calling its own Sensor unit. Therefore the
	 * ProcessedBIR parameter is not present. <br/>
	 * A call to PresetIdentifyPopulation shall be done before this method can
	 * be called in order to establish the population within which the search is
	 * done. <br/>
	 * Refer to the Identify methods in Comparison <br/>
	 * The BSP is responsible for providing the user interface associated with
	 * the enrolment operation as a default. The application may request control
	 * of the GUI "look-and-feel" by providing a GUI callback via the
	 * BSP.subscribeToGUIEvents() method. Since this operation includes capture,
	 * it serializes the use of the sensor device. If two or more applications
	 * are racing for the device, the losers will wait until the operation
	 * completes or the timeout expires. This serialization takes place in all
	 * functions that capture data. The BSP is responsible for serializing. It
	 * may do this by either throwing an exception to indicate that the device
	 * is busy or by queuing requests
	 * 
	 * @param maxFMRrequested
	 * @param subtype
	 *            specifies which subtype (e.g., left/right eye) to capture.
	 *            Null indicates that the value is not provided
	 * @param binning
	 * @param maxResults
	 * @param timeout
	 *            an integer specifying the timeout value (in milliseconds) for
	 *            the operation. If this timeout is reached an exception is
	 *            thrown. This value can be any positive number. -1 value means
	 *            the BSPs default timeout value will be used.
	 * @param options
	 *            requests additional output such as audit data
	 * @return See the Identify methods in Comparison
	 * @throws BioAPIException
	 *             In case of any of the Capture exceptions
	 */
	Vector<Candidate> identifyAggregated(int maxFMRrequested,
			DataTypes.BiometricSubtype subtype, boolean binning,
			int maxResults, int timeout, Vector<DataTypes.ResultOptions> options)
			throws BioAPIException;

	/**
	 * Returns the identification of those BFPs available that are supported by
	 * the BSP in the current session
	 * 
	 * @param unitCategories
	 *            Optionally, the list of categories for which the enumeration
	 *            of the unit schemas is requested
	 * @return List of BFPs that are supported by current BSP
	 * @throws BioAPIException
	 *             Thrown if the method is called after the BSP has been
	 *             detached, or any other error.
	 */
	Vector<BFPListElement> queryBFPs(
			Vector<DataTypes.UnitCategoryType> unitCategories)
			throws BioAPIException;;

	Vector<BFPListElement> queryBFPs() throws BioAPIException;

	/**
	 * Returns Units available by the BSP in the current session
	 * 
	 * @param unitCategories
	 *            Optionally, the list of categories for which the enumeration
	 *            of the unit schemas is requested
	 * @return List of UnitSchemas where each element describes properties of
	 *         each unit available for the current session
	 * @throws BioAPIException
	 *             Thrown if the method is called after the BSP has been
	 *             detached.
	 */
	Vector<UnitSchema> queryUnits(
			Vector<DataTypes.UnitCategoryType> unitCategories)
			throws BioAPIException;

	Vector<UnitSchema> queryUnits() throws BioAPIException;

	/**
	 * This method provides an aggregated functionality. It increases the
	 * functionlality of the Verify method from Comparison (see clause 4.2),
	 * allowing the verify operation in the following cases (in order of the
	 * method list above): <br/>
	 * <ul>
	 * <li>The biometric sample is captured directly by the BSP calling its own
	 * Sensor unit. Therefore the ProcessedBIR parameter is not present.</li>
	 * <li>The biometric sample is supplied but the biometric reference is
	 * declared by its UUID. Therfore the ReferenceKey parameter is added.</li>
	 * <li>Both above mentioned cases at the same time.</li>
	 * </ul>
	 * <br/>
	 * The BSP is responsible for providing the user interface associated with
	 * the enrolment operation as a default. The application may request control
	 * of the GUI "look-and-feel" by providing a GUI callback via the
	 * BSP.subscribeToGUIEvents() method. Since this operation includes capture,
	 * it serializes the use of the sensor device. If two or more applications
	 * are racing for the device, the losers will wait until the operation
	 * completes or the timeout expires. This serialization takes place in all
	 * functions that capture data. The BSP is responsible for serializing. It
	 * may do this by either throwing an exception to indicate that the device
	 * is busy or by queuing requests.
	 * 
	 * @param maxFMRrequested
	 * @param referenceTemplate
	 * @param subtype
	 *            specifies which subtype (e.g., left/right eye) to capture.
	 *            Null indicates that the value is not provided
	 * @param timeout
	 *            an integer specifying the timeout value (in milliseconds) for
	 *            the operation. If this timeout is reached an exception is
	 *            thrown. This value can be any positive number. -1 value means
	 *            the BSPs default timeout value will be used.
	 * @param options
	 *            requests additional output such as audit data
	 * @return See the Verify methods in Comparison
	 * @throws BioAPIException
	 *             In case of any of the Capture exceptions or the Verify
	 *             exceptions
	 */
	boolean verifyAggregated(int maxFMRrequested, BIR referenceTemplate,
			DataTypes.BiometricSubtype subtype, int timeout,
			Vector<DataTypes.ResultOptions> options) throws BioAPIException;

	boolean verifyAggregated(int maxFMRrequested, BIR processedBIR,
			UUID referenceKey, DataTypes.BiometricSubtype subtype, int timeout,
			Vector<DataTypes.ResultOptions> options) throws BioAPIException;

	boolean verifyAggregated(int maxFMRrequested, UUID referenceKey,
			DataTypes.BiometricSubtype subtype, int Timeout,
			Vector<DataTypes.ResultOptions> options) throws BioAPIException;

	/**
	 * This method provides to the BSP the callback functions for the Select,
	 * State and Progress events. If a certain event is not supported, a NULL
	 * value shall be provided for that kind of event. It is not possible to
	 * call this method with the three events set to NULL.
	 * 
	 * @param guiSelectEventCallbackFunction
	 *            specifies address to the callback function for the Select
	 *            Event.
	 * @param guiStateEventCallbackFunction
	 *            specifies address to the callback function for the State
	 *            Event.
	 * @param guiProgressEventCallbackFunction
	 *            specifies address to the callback function for the Progress
	 *            Event.
	 * @throws BioAPIException
	 */
	void subscribeToGUIEvents(
			GUISelectEventListener guiSelectEventCallbackFunction,
			GUIStateEventListener guiStateEventCallbackFunction,
			GUIProgressEventListener guiProgressEventCallbackFunction)
			throws BioAPIException;

	/**
	 * This method clears the callback addresses previously subscribed. After a
	 * call to this function, the BSP shall cease to notify GUI events to the
	 * framework or the application
	 * 
	 * @throws BioAPIException
	 */
	void unsubscribeFromGUIEvents() throws BioAPIException;

}
