package org.bioapi;

import java.util.Vector;
import org.bioapi.data.BFPSchema;
import org.bioapi.data.UnitSchema;

/**
 * Represents the Biometric Function Provider. This interface is composed of the
 * BFP functionality and the integration of those BioAPI_Units supported by the
 * BFP. Within the same BFP only one category of BioAPI_Units shall be included.
 * It also may provide support to the communication with BSPs and the component
 * registry.
 */
public interface BFP {

	BFPSchema getBFPSchema();

	/**
	 * This function creates an Attach Session. There is no requirement that the
	 * unit ID value provided by a BSP as input to this function be the same
	 * unit ID value that the framework provided to the BSP in the original call
	 * to BSPAttach (if any), provided that the two unit ID values identify the
	 * same BioAPI unit.
	 * 
	 * @param unitID
	 *            The ID of that BioAPI Unit defined by a prior call to
	 *            QueryUnits. The UnitID has been filled into the UnitId field
	 *            of each element of the UnitSchema2 by the BFP.
	 * @throws BioAPIException
	 *             Thrown if any the parameters is not valid or any other error
	 *             during attachment.
	 */
	void unitAttach(int unitID) throws BioAPIException;

	/**
	 * This function terminates an Attach Session. There is no requirement that
	 * the unit ID value provided by a BSP as input to this function be the same
	 * unit ID value that the framework provided to the BSP in the original call
	 * to BSPAttach (if any), provided that the two unit ID values identify the
	 * same BioAPI unit.
	 * 
	 * @param unitID
	 *            The ID of that BioAPI Unit defined by a prior call to
	 *            QueryUnits. The UnitID has been filled into the UnitId field
	 *            of each element of the UnitSchema2 by the BFP.
	 * @throws BioAPIException
	 *             Thrown if any internal error has occurred.
	 */
	void unitDetach(int unitID) throws BioAPIException;

	/**
	 * Initializes a BFP. Initialization includes registering the BSP event
	 * handler for the specified BFP and enabling all events. The BSP can choose
	 * to provide an event handler function to receive notification of events.
	 * Many BSP's can independently and concurrently load the same BFP, and each
	 * BSP can establish its own event handler. They will all receive
	 * notification of an event. The same or different event handlers can be
	 * used if a BSP loads multiple BFPs.\n A BSP may establish as many event
	 * handlers as it wishes, for a given BFP, by calling BFPLoad one or more
	 * times for that BFP. An event handler is identified by the address of the
	 * notification.\n When an event occurs in a BFP, the BFP may send an event
	 * notification to the BSP by calling the BSP's event handler.\n If a BSP
	 * has set up multiple event handlers, they shall be called one at a time
	 * (in any order chosen by the BFP) rather than concurrently.\n Event
	 * notification may occur at any time, either during a BSP call (related or
	 * unrelated to the event) or while there is no BSP call in execution. BSP
	 * writers should ensure that all callbacks are properly and safely handled
	 * by the BSP, no matter when the BSP receives them.\n When a BFP is loaded
	 * (bfpLoad), it shall raise an insert event immediately for each present
	 * BioAPI Unit. This will indicate to the BSP that it can go ahead and do a
	 * UnitAttach. If the hardware component for a specific functionality is not
	 * present, the insert event cannot be raised until the hardware component
	 * has been plugged in.\n The function UnitAttach can be invoked multiple
	 * times for each call to BFPLoad. The bfpNotifyCallback defines a callback
	 * used to notify the BSP of events of type BioAPI_EVENT. The BFP shall
	 * retain this information for later use.
	 * 
	 * @param bfpNotifyCallback
	 *            defines a callback used to notify the BioAPI Framework (or the
	 *            application in a framework-free implementation) of events in
	 *            any ongoing attached session.
	 * @param bfpGUIProgressEventListener
	 *            defines a callback used to notify the BioAPI Framework (or the
	 *            application in a framework-free implementation) of events in
	 *            any ongoing attached session.
	 * @throws BioAPIException
	 *             Thrown if any internal error has occurred. BioAPIException
	 *             (see Clause 8.1)
	 */
	void bfpLoad(BFPEventListener bfpEventListener,
			BFPGUIProgressEventListener bfpGUIProgressEventListener)
			throws BioAPIException;

	/**
	 * This function returns a list of BioAPI Unit schemas, which are managed by
	 * the given BFP and are currently in the inserted state. <br/>
	 * This function shall only be called after BFPLoad has been called for the
	 * specified BFP. <br/>
	 * There is no requirement that a unit ID, returned by this function for a
	 * given BioAPI unit, be made available with the same unit ID value by the
	 * BSP to the framework. A BSP is free to translate any unit ID value
	 * (provided by a BFP) into a different unit ID value before providing it to
	 * the framework. The purpose of such a translation would be to avoid the
	 * existence of duplicate unit IDs within the scope of the BSP, which might
	 * happen when a BSP is using two or more BFPs of the same category, or when
	 * a BSP is using a BSFP while also directly managing biometric sensors.
	 * 
	 * @return List of UnitSchemas where each element describes properties of
	 *         each unit available for the current session
	 * @throws BioAPIException
	 *             Thrown if the method is called after the BFP has been
	 *             detached.
	 */
	Vector<UnitSchema> queryUnits() throws BioAPIException;

}
