package org.bioapi;

import java.util.Vector;
import org.bioapi.data.BIR;
import org.bioapi.data.Candidate;
import org.bioapi.data.DataTypes.ResultOptions;
import org.bioapi.data.IdentifyPopulation;
import org.bioapi.data.UnitSchema;

/**
 * This interface includes all properties and methods needed to perform
 * biometric comparison functionality. Some properties are defined to return the
 * results of the Verify methods, securing their values as no modification is
 * allowed publicly. Methods are overloaded to allow the comparison of single
 * samples, or to adapt the comparison algorithm internally by the use of a list
 * of BIRs in addition to the sample BIR to be compared. </br> Within this unit,
 * the term FMR is used. FMR represents the False Match Rate as a 32-bit integer
 * value (N) that indicates a probable False Matching Rate of N/(231-1). The
 * larger the value, the worse the result. Negative values are used to signal
 * exceptional conditions. The only negative value currently defined is minus
 * one. For security reasons no methods to modify the existing FMR object are
 * provided. </br> Methods and properties from this Interface shall not be
 * accessible by the application or the framework, as this interface is only
 * intended to be included in a BSP or a BFP.
 */
public interface Comparison {

	UnitSchema getUnitSchema();

	void setUnitSchema(UnitSchema unitSchema);

	byte[] getACBioInstance();

	int getFMRAchieved();

	BIR getAdaptedBIR();

	byte[] getPayload();

	/**
	 * Sends control data to the BioAPI_Unit and receives status or operation
	 * data from there. The content of the parameters and the output will be
	 * specified in the related interface specification of this BioAPI_Unit
	 * 
	 * @param controlCode
	 *            The function code in the BioAPI_Unit to be called
	 * @param inputData
	 *            Buffer containing the data to be sent to the BioAPI unit
	 *            related to the given ControlCode
	 * @return Data buffer containing the data received from the BioAPI_Unit
	 *         after processing the function indicated by the ControlCode. If no
	 *         memory block has been allocated by the function the value shall
	 *         be NULL.
	 * @throws BioAPIException
	 *             If case of any error
	 */
	byte[] controlUnit(int controlCode, byte[] inputData)
			throws BioAPIException;

	/**
	 * Performs biometric identification of the existing biometric sample. This
	 * function performs an identification (1-to-many) comparison between a
	 * processed BIR and a set of reference BIRs. The input is the processed BIR
	 * captured specifically for this identification. The population that the
	 * comparison takes place against shall be previously stated by calling
	 * PresetIdentifyPopulation.
	 * 
	 * @param maxFMRrequested
	 *            the requested FMR criterion for successful identification
	 *            (i.e., the comparison threshold).
	 * @param processedBIR
	 *            the BIR to be identified
	 * @param binning
	 *            a boolean indicating whether binning is on or off. Binning is
	 *            a search optimization technique that the BSP may employ. It is
	 *            based on searching a subset of the population according to the
	 *            intrinsic characteristics of the biometric data. While it may
	 *            improve the speed of the compare operation, it may also
	 *            increase the probability of missing a candidate (due to the
	 *            possibility of mis-binning and as a result, searching a bin
	 *            which should, but does not, contain the matching BIR).
	 * @param maxResults
	 *            A value of zero is a request for all canddates.
	 * @param timeout
	 *            an integer specifying the timeout value (in milliseconds) for
	 *            the operation. If this timeout is reached an exception is
	 *            thrown. This value can be any positive number. A -1 value
	 *            means the BSPs default timeout value will be used
	 * @return The list of CandDates that represents the result of the biometric
	 *         operation
	 * @throws BioAPIException
	 *             If input parameters are invalid or operation fails due to an
	 *             error.
	 */
	Vector<Candidate> identify(int maxFMRrequested, BIR processedBIR,
			boolean binning, int maxResults, int timeout)
			throws BioAPIException;

	/**
	 * Performs biometric identification of the existing biometric sample. This
	 * function performs an identification (1-to-many) comparison between a
	 * processed BIR and a set of reference BIRs. The input is the processed BIR
	 * captured specifically for this identification. The population that the
	 * comparison takes place against shall be previously stated by calling
	 * PresetIdentifyPopulation.
	 * 
	 * @param maxFMRrequested
	 *            the requested FMR criterion for successful identification
	 *            (i.e., the comparison threshold).
	 * @param processedBIR
	 *            the BIR to be identified
	 * @param auxiliaryBIRs
	 *            the list of auxiliary BIRs used to improve the performance of
	 *            the comparison algorithm
	 * @param binning
	 *            a boolean indicating whether binning is on or off. Binning is
	 *            a search optimization technique that the BSP may employ. It is
	 *            based on searching a subset of the population according to the
	 *            intrinsic characteristics of the biometric data. While it may
	 *            improve the speed of the compare operation, it may also
	 *            increase the probability of missing a candidate (due to the
	 *            possibility of mis-binning and as a result, searching a bin
	 *            which should, but does not, contain the matching BIR).
	 * @param maxResults
	 *            A value of zero is a request for all canddates.
	 * @param timeout
	 *            an integer specifying the timeout value (in milliseconds) for
	 *            the operation. If this timeout is reached an exception is
	 *            thrown. This value can be any positive number. A -1 value
	 *            means the BSPs default timeout value will be used
	 * @return The list of CandDates that represents the result of the biometric
	 *         operation
	 * @throws BioAPIException
	 *             If input parameters are invalid or operation fails due to an
	 *             error.
	 */
	Vector<Candidate> identify(int maxFMRrequested, BIR processedBIR,
			Vector<BIR> auxiliaryBIRs, boolean binning, int maxResults,
			int timeOut) throws BioAPIException;

	/**
	 * Provides the population of BIRs to the comparison unit. After this method
	 * is invoked successfully, a BSP can call identify(). The BSP keeps this
	 * setting in effect until the presetIdentifyPopulation() is invoked with
	 * different setting or either the BSP or the session is terminated.
	 * 
	 * @param population
	 *            the population of BIRs against which the identification is
	 *            performed
	 * @throws BioAPIException
	 *             If input parameters are invalid or operation fails due to an
	 *             error.
	 */
	void presetIdentifyPopulation(IdentifyPopulation population)
			throws BioAPIException;

	/**
	 * Performs biometric verification of an existing biometric sample. This
	 * function performs a verification (1-to-1) comparison between two BIRs:
	 * the input BIR and the biometric reference. The input BIR is the processed
	 * BIR constructed specifically for this verification. The reference
	 * template was created at enrolment. The application shall request a
	 * maximum FMR value criterion (threshold) for a successful comparison.
	 * </br> The Boolean output of the method indicates whether the verification
	 * was successful or not, and the internal properties (fmrAchieved,
	 * adaptedBIR and payload) are updated. By setting the RequestAdaptedBir
	 * option the application can request that a BIR be constructed by adapting
	 * the reference template using the input processed BIR. </br> If the
	 * comparison is successful, an attempt may be made to adapt the reference
	 * template with information taken from the input BIR. (Not all BSPs perform
	 * adaptation). The resulting adapted BIR should now be considered an
	 * optimal enrolment template (it is up to the application whether it uses
	 * or discards this data.). It is important to note that adaptation may not
	 * occur in all cases. </br> If a payload is associated with the reference
	 * template, the payload may be returned upon successful verification if the
	 * achieved FMR is sufficiently stringent; this is controlled by the policy
	 * of the BSP and specified in its schema. The return of payload is
	 * requested by setting the requestPayload option. </br> Access to the
	 * different ways of the results is provided by the Get functions of this
	 * interface. This is done in this way so that the results cannot be
	 * modified outside of the Comparison Unit. </br> This method is overloaded
	 * with an additional parameter stating a list of auxiliary BIRs that may
	 * help to optimize the behaviour of the comparison algorithm.
	 * 
	 * @param maxFMRrequested
	 *            the requested FMR criterion for successful verification (i.e.,
	 *            the comparison threshold).
	 * @param processedBIR
	 *            the BIR to be identified
	 * @param referenceTemplate
	 *            the BIR to be verified against
	 * @param options
	 *            requests additional output such as adapted BIR and/or the
	 *            payload
	 * @return The boolean decision that represents the result of the biometric
	 *         operation
	 * @throws BioAPIException
	 *             If input parameters are invalid or operation fails due to an
	 *             error.
	 */
	boolean verify(int maxFMRrequested, BIR processedBIR,
			BIR referenceTemplate, Vector<ResultOptions> options)
			throws BioAPIException;

	/**
	 * Performs biometric verification of an existing biometric sample. This
	 * function performs a verification (1-to-1) comparison between two BIRs:
	 * the input BIR and the biometric reference. The input BIR is the processed
	 * BIR constructed specifically for this verification. The reference
	 * template was created at enrolment. The application shall request a
	 * maximum FMR value criterion (threshold) for a successful comparison.
	 * </br> The Boolean output of the method indicates whether the verification
	 * was successful or not, and the internal properties (fmrAchieved,
	 * adaptedBIR and payload) are updated. By setting the RequestAdaptedBir
	 * option the application can request that a BIR be constructed by adapting
	 * the reference template using the input processed BIR. </br> If the
	 * comparison is successful, an attempt may be made to adapt the reference
	 * template with information taken from the input BIR. (Not all BSPs perform
	 * adaptation). The resulting adapted BIR should now be considered an
	 * optimal enrolment template (it is up to the application whether it uses
	 * or discards this data.). It is important to note that adaptation may not
	 * occur in all cases. </br> If a payload is associated with the reference
	 * template, the payload may be returned upon successful verification if the
	 * achieved FMR is sufficiently stringent; this is controlled by the policy
	 * of the BSP and specified in its schema. The return of payload is
	 * requested by setting the requestPayload option. </br> Access to the
	 * different ways of the results is provided by the Get functions of this
	 * interface. This is done in this way so that the results cannot be
	 * modified outside of the Comparison Unit. </br> This method is overloaded
	 * with an additional parameter stating a list of auxiliary BIRs that may
	 * help to optimize the behaviour of the comparison algorithm.
	 * 
	 * @param maxFMRrequested
	 *            the requested FMR criterion for successful verification (i.e.,
	 *            the comparison threshold).
	 * @param processedBIR
	 *            the BIR to be identified
	 * @param referenceTemplate
	 *            the BIR to be verified against
	 * @param auxiliaryBIRs
	 *            the list of auxiliary BIRs used to improve the performance of
	 *            the comparison algorithm
	 * @param options
	 *            requests additional output such as adapted BIR and/or the
	 *            payload
	 * @return The boolean decision that represents the result of the biometric
	 *         operation
	 * @throws BioAPIException
	 *             If input parameters are invalid or operation fails due to an
	 *             error.
	 */
	boolean verify(int maxFMRrequested, BIR processedBIR,
			BIR referenceTemplate, Vector<BIR> auxiliaryBIRs,
			Vector<ResultOptions> options) throws BioAPIException;

}
