package org.bioapi;

import java.util.UUID;
import org.bioapi.data.BFPSchema;
import org.bioapi.data.BSPSchema;

/**
 * Represents the interface to the component registry. The Installer adds
 * deletes of modifies BSP and BFP records in the component registry managed by
 * the framework. <br/>
 * The way the component registry is implemented is dependent on the developer,
 * as the components are managed from the provided methods. Further information
 * about the information that the component registry shall contain internally
 * (e.g. a framework, BSP, Units schemas) is available at the "Component
 * registry interface" clause in ISO/IEC 19784-1.
 */
public interface ComponentRegistry {

	/**
	 * Installs or updates the references to a BFP in the component registry.
	 * This function is handled internally within the BioAPI Framwork and is not
	 * passed through to a BFP.
	 * 
	 * @param bfpSchema
	 *            specifies the information on the BFP to be installed or
	 *            updated.
	 * @param update
	 *            If true, an update of an existing BFP is performed (i.e. if
	 *            such BFP is not yet installed, it will return an
	 *            ERR_COMPONENT_NOT_REGISTERED BioAPIException). If false, the
	 *            installation refers to a new BFP. If the BFP is already
	 *            installed an ERR_COMPONENT_ALREADY_REGISTERED BioAPIException
	 *            is thrown.
	 * @throws BioAPIException
	 *             Thrown if any error during installation.
	 */
	void installBFP(BFPSchema bfpSchema, boolean update) throws BioAPIException;

	/**
	 * Installs or updates the references to a BSP in the component registry.
	 * This function is handled internally within the BioAPI Framwork and is not
	 * passed through to a BSP.
	 * 
	 * @param bspSchema
	 *            specifies the information on the BSP to be installed or
	 *            updated.
	 * @param update
	 *            If true, an update of an existing BSP is performed (i.e. if
	 *            such BSP is not yet installed, it will return an
	 *            ERR_COMPONENT_NOT_REGISTERED BioAPIException). If false, the
	 *            installation refers to a new BSP. If the BSP is already
	 *            installed an ERR_COMPONENT_ALREADY_REGISTERED BioAPIException
	 *            is thrown.
	 * @throws BioAPIException
	 *             Thrown if any error during installation.
	 */
	void installBSP(BSPSchema bspSchema, boolean update) throws BioAPIException;

	/**
	 * This function uninstalls a BFP by removing references to that BFP in the
	 * component registry. <br/>
	 * This function is handled internally within the BioAPI Framework and is
	 * not passed through to a BFP.
	 * 
	 * @param bfpUUID
	 *            specifies BFP to be uninstalled.
	 * @throws BioAPIException
	 *             Thrown if any error.
	 */
	void uninstallBFP(UUID bfpUUID) throws BioAPIException;

	/**
	 * This function uninstalls a BSP by removing references to that BSP in the
	 * component registry. <br/>
	 * This function is handled internally within the BioAPI Framework and is
	 * not passed through to a BSP.
	 * 
	 * @param bspUUID
	 *            specifies BSP to be uninstalled.
	 * @throws BioAPIException
	 *             Thrown if any error.
	 */
	void uninstallBSP(UUID bspUUID) throws BioAPIException;

}
