package org.bioapi;

import java.util.UUID;
import java.util.Vector;
import org.bioapi.data.ACBioParameters;
import org.bioapi.data.BFPListElement;
import org.bioapi.data.BFPSchema;
import org.bioapi.data.BSPSchema;
import org.bioapi.data.DataTypes.EventKind;
import org.bioapi.data.FrameworkSchema;
import org.bioapi.data.SecurityProfileType;
import org.bioapi.data.UnitListElement;
import org.bioapi.data.UnitSchema;

/**
 * Represents the biometric system. The biometric system is the hierarchical
 * assembly which root node is the Framework component. The Framework controls
 * biometric service providers (BSPs) that drive concrete biometric sensor units
 * and implement various biometric technologies. To provide necessary services
 * to the Framework BSPs use biometric function provider modules that are
 * libraries implementing biometric algorithms and/or code that manages sensors
 * hardware. Along with BSPs, another subordinate of the Framework is component
 * registry that stores the information about BSPs and BFPs that areavailable in
 * the biometric system.
 * 
 * @author See CREDITS.txt
 */
public interface Framework extends ComponentRegistry {

	FrameworkSchema getFrameworkSchema();

	Vector<AttachSession> getAttachedSessionList();

	/**
	 * Initialize a BSP attach session and verifies that the version of the BSP
	 * expected by the application is compatible with the version on the system.
	 * Both normal and secure attach sessions are supported. (The BSP shall
	 * indicate within the BSP Schema if it supports secure attach sessions or
	 * not.) The caller shall specify a list of zero or more BioAPI Units that
	 * the BSP is to use in the attach session being created, and also specify
	 * the security profile information to be used in biometric operations
	 * afterwards. <br/>
	 * This function shall only be called (for a given BSP UUID) if there is at
	 * least one call to bspLoad (for that BSP UUID) for which a corresponding
	 * call to bspUnload has not yet been made. The function bspAttach can be
	 * invoked multiple times for each call to bspLoad (prior to a call to
	 * bspUnload), for the same BSP, creating multiple invocations of that BSP. <br/>
	 * If a BioAPI unit used in an attach session becomes unavailable, the
	 * attach session shall be invalDated. When an attach session has been
	 * invalDated, the only function that may be legally called specifying that
	 * attach session is bspDetach. All other functions shall return an
	 * ERR_INVALID_ATTACH_SESSION BioAPIException. <br/>
	 * If a BSP supports events, it may generate a "remove" event when a BioAPI
	 * unit becomes unavailable. If the application has set up an event handler
	 * and has not disabled "remove" event notifications, it will receive the
	 * "remove" event notification, and will be able to infer that all the
	 * attach sessions (if any) that were using the removed BioAPI unit are now
	 * invalid. The application can then either terminate those attach sessions
	 * by calling bspDetach, or be prepared to receive an
	 * ERR_INVALID_ATTACH_SESSION error from a subsequent function call, and
	 * then call bspDetach. </br> NOTE: If a BioAPI unit was implicitly selected
	 * into an attach session (either through the use of DONT_CARE or by not
	 * including the category of that unit in the list provided as input to
	 * bspAttach), the application may not know which attach sessions were using
	 * the BioAPI unit that has been removed, and thus may not be able to
	 * terminate those attach sessions without first getting
	 * ERR_INVALID_ATTACH_SESSION errors from subsequent function calls.
	 * 
	 * @param bspUUID
	 *            The identifier of the BSP that is going to be attached to that
	 *            session.
	 * @param bioAPIVersion
	 *            The major and minor version number of the BioAPI specification
	 *            that the application is expecting the BSP to support. The BSP
	 *            shall determine whether it is compatible with the required
	 *            version.
	 * @param acBioParameters
	 *            Parameters about ACBio generation. A NULL pointer shall be set
	 *            if no generation of ACBio instances is necessary.
	 * @param unitList
	 *            a pointer to a buffer containing a list of UnitListElement
	 *            structures indicating to the BSP which BioAPI Units (supported
	 *            by the BSP) it is to use for this attach session. The
	 *            structures contain the ID and category of each BioAPI Unit.
	 *            One of the following will be specified for each category of
	 *            BioAPI Unit: <br/>
	 *            <ul>
	 *            <li>Selection of a specific BioAPI Unit: The particular BioAPI
	 *            Unit to be used in this attach session is specified by
	 *            inclusion of its ID and category.</li>
	 *            <li>Selection of any BioAPI Unit: When the UnitID is set to
	 *            BioAPIDontCare (i.e. 0x00000000) in a particular element, the
	 *            BSP will choose which BioAPI Unit of that category to use, or
	 *            will give an error return if it does not support any BioAPI
	 *            Units of that category. If a particular category is not
	 *            listed, the BSP will likewise choose a BioAPI Unit of that
	 *            category to use if it supports a BioAPI Unit of that category
	 *            (however, there is no error return if it does not).</li>
	 *            <li>Selection of no BioAPI Unit: When the UnitID is set to
	 *            BioAPIDontInclude (i.e. 0xFFFFFFFF), the BSP will explicitly
	 *            not attach a BioAPI Unit of the given category, even if it
	 *            supports one of that category.</li><br/>
	 *            NOTE: Any subsequent calls requiring use of a BioAPI Unit of
	 *            this category will fail with an error return.
	 *            </ul>
	 * 
	 * @param securityProfileList
	 *            a pointer to a buffer containing a list of
	 *            BioAPI_SECURITY_PROFILE structures which contain security
	 *            information to be used in security operations by BioAPI Units.
	 *            The order of this list shall keep that of the UnitList. If a
	 *            secure attach is not used, this parameter shall be set to
	 *            NULL.
	 * @return The created attached session so that the application can then use
	 *         it in order to call the functionality of that BSP.
	 * @throws BioAPIException
	 *             Thrown if any the parameters is not valid or any other error
	 *             during attachment.
	 */
	AttachSession bspAttach(UUID bspUUID, String bioAPIVersion,
			ACBioParameters acBioParameters, Vector<UnitListElement> unitList,
			Vector<SecurityProfileType> securityProfileList)
			throws BioAPIException;

	/**
	 * This function detaches the biometric application from the BSP invocation. <br/>
	 * At this time, all BSP allocated resources associated with the BSP attach
	 * session shall be freed or released or invalDated. At this time, all set
	 * callbacks associated with the BSP attach session shall become invalid. <br/>
	 * This function shall only be called after BSPAttach has been called, and
	 * shall not be called more than once for the same BSP handle created by the
	 * call to bspAttach.
	 * 
	 * @param sessionAttached
	 *            Identifies the BSP to initialize.
	 * @throws BioAPIException
	 *             Thrown if the framework has been terminated, there is no
	 *             attached session or any other error.
	 */
	void bspDetach(AttachSession sessionAttached) throws BioAPIException;

	/**
	 * Initializes a BSP using the BSP.bspLoad. Initialization includes enabling
	 * all events except those that were disabled by the most recent call to
	 * EnableEventNotifications. The application can choose to provide an event
	 * handler function to receive notification of events. Many applications can
	 * independently and concurrently load the same BSP, and each application
	 * can establish its own event handler. They will all receive notification
	 * of an event. The same or different event handlers can be used if an
	 * application loads multiple BSPs. <br/>
	 * An application may establish as many event handlers as it wishes, for a
	 * given BSP, by calling BioAPI_BSPLoad one or more times for that BSP. An
	 * event handler is identified by a combination of address and context. <br/>
	 * When an event occurs in a BSP, the BSP may send an event notification to
	 * the Framework by calling the Framework's event handler. <br/>
	 * When the Framework receives an event notification from a BSP, it shall
	 * send one notification to each event handler established by each
	 * application for which that event notification is enabled for that BSP.
	 * Therefore, a single event notification callback made from a BSP to the
	 * Framework may result in zero or more callbacks made by the Framework to
	 * zero or more applications. <br/>
	 * When the framework receives an event notification from a BSP, it shall
	 * call all the event handlers established by each application for that BSP.
	 * If an application has set up multiple event handlers, they shall be
	 * called one at a time (in any order chosen by the Framework) rather than
	 * concurrently. <br/>
	 * Event notification may occur at any time, either during a BioAPI call
	 * (related or unrelated to the event) or while there is no BioAPI call in
	 * execution. Application writers should ensure that all callbacks are
	 * properly and safely handled by the application, no matter when the
	 * application receives them. <br/>
	 * NOTE: This usually requires the use of thread synchronization techniques
	 * and discipline in the actions performed by the application code placed <br/>
	 * There is a "use count" on the establishment of event handlers; they have
	 * to be de-established (by bspUnload) as many times as they were
	 * established. When the BioAPI Framework calls BSPLoad, it receives from
	 * the BSP an â€œinsertâ€� event notification for each available BioAPI unit
	 * (of each category). If the biometric application has provided an event
	 * handler in the call to bspLoad and has not disabled â€œinsertâ€� event
	 * notifications, the Framework will call back, in turn, the application's
	 * event handler. Insert notifications can be disabled by calling the
	 * function EnableEventNotifications. This will indicate to the biometric
	 * application that it can go ahead and do a BSPAttach. If the hardware
	 * component for a specific functionality is not present, the "insert" event
	 * cannot be raised until the hardware component has been plugged in. in
	 * event handlers. <br/>
	 * This function shall only be called if there is at least one call to Init
	 * for which a corresponding call to Terminate has not yet been made. The
	 * function BSPAttach can be invoked multiple times for each call to
	 * bspLoad. <br/>
	 * The bspLoad function is not to be called unless the BSP has been
	 * installed using installBSP. A determination of installed BSPs can be made
	 * through a call to enumBSPs.
	 * 
	 * @param bspID
	 *            Identifies the BSP to initialize.
	 * @param bfpEventListener
	 *            the event notification function provided by the caller. This
	 *            defines the callback for event notifications from the loaded
	 *            (and later attached) biometric service provider.
	 * @param bfpGUIProgressEventListener
	 *            the event notification function provided by the caller. This
	 *            defines the callback for event notifications from the loaded
	 *            (and later attached) biometric service provider.
	 * @param bfpEnumerationListener
	 *            defines a callback used to identify the BFPs installed in the
	 *            framework (or in the application in case of a framework-free
	 *            deployment)
	 * @param context
	 *            generic pointer to context information. When the selected
	 *            biometric service provider raises an event, this value is
	 *            passed as input to the event handler specified by
	 *            notifyCallback.
	 * @throws BioAPIException
	 *             Thrown if the framework has been terminated or any other
	 *             error.
	 */
	void bspLoad(UUID bspID, BFPEventListener bfpEventListener,
			BFPGUIProgressEventListener bfpGUIProgressEventListener,
			BFPEnumerationListener bfpEnumerationListener, String context)
			throws BioAPIException;

	/**
	 * De-registers event notification callbacks for the caller identified by
	 * bspUuid. <br/>
	 * bspUnload is the analogue call to bspLoad. If all callbacks registered
	 * with BioAPI are removed, then BioAPI unloads (for that biometric
	 * application) the BSP that was loaded by calls to bspLoad. <br/>
	 * The BioAPI Framework uses the three input parameters to uniquely identify
	 * registered callbacks. <br/>
	 * This function shall only be called (for a given BSP UUID) if there is at
	 * least one call to bspLoad (for that BSP UUID) for which a corresponding
	 * call to this function has not yet been made. <br/>
	 * This function should not be called by the application if there is a call
	 * to bspAttach for which a corresponding call to bspDetach (for a given BSP
	 * handle) has not yet been made. However, should this function be called
	 * while the BSP is still attached, then for each call to bspAttach without
	 * a corresponding call to bspDetach, the actions relative to the missing
	 * corresponding call to bspDetach shall be implicitly performed by the
	 * BioAPI Framework (as though the corresponding function had been called at
	 * that time), followed by the actions relative to bspUnload, (i.e., the
	 * Framework will detach the BSP prior to unloading it). <br/>
	 * This includes the case in which the actions relative to a missing call to
	 * bspUnload are implicitly performed by the BioAPI Framework during a call
	 * to terminate.
	 * 
	 * @param bspID
	 *            Identifies the BSP to initialize.
	 * @param context
	 *            generic pointer to context information. When the selected
	 *            biometric service provider raises an event, this value is
	 *            passed as input to the event handler specified by
	 *            notifyCallback.
	 * @throws BioAPIException
	 *             Thrown if the framework has been terminated or any other
	 *             error.
	 */
	void bspUnload(UUID bspID, String context) throws BioAPIException;

	/**
	 * Enables the events specified by the Events parameter and coming from the
	 * BSP identified by the BSP UUID, and disables all other events coming from
	 * that BSP. <br/>
	 * Events are enabled or disabled only for the application that calls this
	 * function. If there are other applications using the BioAPI Framework or
	 * the specified BSP at the same time, those applications will not be
	 * affected by the call to enableEventNotifications. <br/>
	 * This function can be called at any time after BioAPI_Init, even before
	 * the specified BSP has been loaded. <br/>
	 * The Event parameter established by a call to this function remains in
	 * effect until this function is called again for the same BSP. <br/>
	 * This function shall only be called if there is at least one call to Init
	 * for which a corresponding call to Terminate has not yet been made. <br/>
	 * This function is handled internally within the BioAPI Framework and is
	 * not passed through to a BSP.
	 * 
	 * @param bspID
	 *            Identifies the BSP to initialize.
	 * @param events
	 *            the list of events where the notifications are expected to
	 *            start being supported.
	 * @throws BioAPIException
	 *             Thrown if the framework has been terminated or any other
	 *             error.
	 */
	void enableEventNotifications(UUID bspID, Vector<EventKind> events)
			throws BioAPIException;

	/**
	 * This function provides information about all BFPs currently installed in
	 * the component registry. <br/>
	 * This function shall only be called if there is at least one call to Init
	 * for which a corresponding call to Terminate has not yet been made. <br/>
	 * This function is handled internally within the BioAPI Framework and is
	 * not passed through to any BSP.
	 * 
	 * @return List of BFPSchemas of the BFPs available in the system
	 * @throws BioAPIException
	 *             Thrown if the method is called after framework is terminated.
	 */
	Vector<BFPSchema> enumBFPs() throws BioAPIException;

	/**
	 * Provides information about all BSPs currently installed in the component
	 * registry. It fills the array with the BSP schemas of all installed BSPs. <br/>
	 * This function shall only be called if there is at least one call to Init
	 * for which a corresponding call to Terminate has not yet been made. <br/>
	 * This function is handled internally within the BioAPI Framework and is
	 * not passed through to any BSP.
	 * 
	 * @return List of bspSchema where each element describes properties of one
	 *         BSP available in the biometric system
	 * @throws BioAPIException
	 *             Thrown if the method is called after framework is terminated,
	 *             or in case any other error.
	 */
	Vector<BSPSchema> enumBSPs() throws BioAPIException;

	/**
	 * Initializes the BioAPI Framework and verifies that the version of the
	 * BioAPI Framework expected by the application is compatible with the
	 * version of the BioAPI Framework on the system. This function should be
	 * called at least once by the application
	 * 
	 * @param version
	 *            specifies the major and minor version number of the BioAPI
	 *            specification that the biometric application is compatible
	 *            with
	 * @throws BioAPIException
	 */
	void init(String version) throws BioAPIException;

	/**
	 * Returns a list of BFPs which are currently installed in the component
	 * registry and supported by the BSP identified by the given BSP UUID. It
	 * performs the following actions (in order): <br/>
	 * <ol>
	 * <li>determines which among all currently installed BFPs are supported by
	 * the BSP</li>
	 * <li>fills the array with identification information (category and UUID)
	 * about the BFPs determined in (1)</li>
	 * </ol>
	 * <br/>
	 * NOTE: This method calls the corresponding BSP.queryBFPs method. <br/>
	 * Additional information about the supported BFPs can be retrieved by
	 * calling enumBFPs and analyzing the BFPSchemaArray at the matching BFP
	 * UUIDs. <br/>
	 * This function shall only be called after BSPLoad has been called for the
	 * specified BSP, and shall not be called after bspUnload has been called
	 * for the BSP.
	 * 
	 * @param bspUUID
	 *            The identifier of the BSP for which the BFP information is to
	 *            be returned.
	 * @return The list BFPs supported by the BSP.
	 * @throws BioAPIException
	 *             Thrown if any error.
	 */
	Vector<BFPListElement> queryBFPs(UUID bspUUID) throws BioAPIException;

	/**
	 * Provides information about all BioAPI Units that are managed directly or
	 * indirectly by the BSP identified by the given BSP UUID and are currently
	 * in the inserted state (i.e. after bspLoad). It performs the following
	 * actions (in order): <br/>
	 * <ol>
	 * <li>determines the set of BioAPI Units that are managed directly or
	 * indirectly by the BSP and are currently in the inserted state</li>
	 * <li>fills the list with the unit schemas of all BioAPI Units determined
	 * in (1)</li>
	 * </ol>
	 * <br/>
	 * NOTE: This method calls BSP.queryUnits for the referenced BSP <br/>
	 * This function shall only be called after bspLoad has been called for the
	 * specified BSP, and shall not be called after bspUnload has been called
	 * for the BSP.
	 * 
	 * @param bspUUID
	 *            The identifier of the BSP for which the unit information is to
	 *            be returned.
	 * @return The list UnitSchema2 from units supported by the BSP.
	 * @throws BioAPIException
	 *             Thrown if any error.
	 */
	Vector<UnitSchema> queryUnits(UUID bspUUID) throws BioAPIException;

	/**
	 * Terminates a biometric application's use of the BioAPI Framework. <br/>
	 * This function shall only be called if there is at least one successful
	 * call to init() for which a corresponding call to this function has not
	 * yet been made. terminate() shall not perform any actions if there are any
	 * outstanding calls to Init (i.e., calls that have not been followed by
	 * corresponding calls to terminate). <br/>
	 * This function should not be called by the application if there is a call
	 * to bspLoad for which a corresponding call to bspUnload (for a given BSP
	 * UUID) has not yet been made. However, should this function be called
	 * while there are still BSPs loaded, then for each call to bspLoad without
	 * a corresponding call to bspUnload, the actions relative to the missing
	 * corresponding call to bspUnload shall be implicitly performed by the
	 * Framework (as though the corresponding function had been called at that
	 * time), followed by the actions relative to terminate (i.e, the Framework
	 * shall unload any BSPs prior to terminating). <br/>
	 * This function is handled internally within the BioAPI Framework and is
	 * not passed through to a BSP, except where a bspUnload is implied, as
	 * specified above.
	 * 
	 * @throws BioAPIException
	 */
	void terminate() throws BioAPIException;

}
