package org.bioapi;

import java.util.Vector;
import org.bioapi.data.BIR;
import org.bioapi.data.DataFormat;
import org.bioapi.data.UnitSchema;

/**
 * Represents the group of biometric operation that are available when the unit
 * of processing category is being used. </br> Methods and properties from this
 * Interface shall not be accessible by the application or the framework, as
 * this interface is only intended to be included in a BSP or a BFP.
 */
public interface Processing {

	UnitSchema getUnitSchema();

	void setUnitSchema(UnitSchema unitSchema);

	byte[] getACBioInstance();

	/**
	 * Sends control data to the BioAPI_Unit and receives status or operation
	 * data from there. The content of the parameters and the output will beç
	 * specified in the related interface specification of this BioAPI_Unit
	 * 
	 * @param controlCode
	 *            The function code in the BioAPI_Unit to be called
	 * @param inputData
	 *            Buffer containing the data to be sent to the BioAPI unit
	 *            related to the given ControlCode
	 * @return Data buffer containing the data received from the BioAPI_Unit
	 *         after processing the function indicated by the ControlCode. If no
	 *         memory block has been allocated by the function the value shall
	 *         be NULL.
	 * @throws BioAPIException
	 *             If case of any error
	 */
	byte[] controlUnit(int controlCode, byte[] inputData)
			throws BioAPIException;

	/**
	 * Takes a BIR or a list of BIRs containing biometric data for the purpose
	 * of creating a new enrolment template. A new BIR is constructed from the
	 * captured BIR, and it may perform an update based on an existing reference
	 * template. The optional input reference template is provided for use in
	 * creating a new template if the BSP supports this capability
	 * 
	 * @param capturedBIR
	 *            The captured BIR
	 * @param referenceTemplate
	 *            optionally, the existing template to be updated
	 * @param outputFormat
	 *            Specifies which BDB format to use for the returned processed
	 *            BIR, if the BSP supports more than one format. A null value
	 *            indicates that the BSP is to select the format
	 * @param payload
	 *            a payload that will be stored by the BSP.
	 * @return Returns the BIR object that represents the result of processing
	 * @throws BioAPIException
	 *             If the input parameters are invalid, the format is not
	 *             supported or operation fails due to error.
	 */
	BIR createTemplate(BIR capturedBIR, BIR referenceTemplate,
			DataFormat outputFormat, byte[] payload) throws BioAPIException;

	/**
	 * Takes a BIR or a list of BIRs containing biometric data for the purpose
	 * of creating a new enrolment template. A new BIR is constructed from the
	 * captured BIR, and it may perform an update based on an existing reference
	 * template. The optional input reference template is provided for use in
	 * creating a new template if the BSP supports this capability
	 * 
	 * @param capturedBIRs
	 *            The list of captured BIRs
	 * @param referenceTemplate
	 *            optionally, the existing template to be updated
	 * @param outputFormat
	 *            Specifies which BDB format to use for the returned processed
	 *            BIR, if the BSP supports more than one format. A null value
	 *            indicates that the BSP is to select the format
	 * @param payload
	 *            a payload that will be stored by the BSP.
	 * @return Returns the BIR object that represents the result of processing
	 * @throws BioAPIException
	 *             If the input parameters are invalid, the format is not
	 *             supported or operation fails due to error.
	 */
	BIR createTemplate(Vector<BIR> capturedBIRs, BIR referenceTemplate,
			DataFormat outputFormat, byte[] payload) throws BioAPIException;

	/**
	 * Processes the biometric sample captured, in order to create a processed
	 * biometric sample for the purpose of either verification or
	 * identification. The overloaded method enables implementations that
	 * require the input of auxiliary data to process the operation.
	 * 
	 * @param capturedBIR
	 *            the captured BIR
	 * @param outputFormat
	 *            specifies which BDB format to use for the returned processed
	 *            BIR, if the BSP supports more than one format. null indicates
	 *            that the BSP is to select the format
	 * @return The BIR object that represents the result of processing
	 * @throws BioAPIException
	 *             If the input parameters are invalid or operation fails due to
	 *             error.
	 */
	BIR process(BIR capturedBIR, DataFormat outputFormat)
			throws BioAPIException;

	/**
	 * Processes the biometric sample captured, in order to create a processed
	 * biometric sample for the purpose of either verification or
	 * identification. The overloaded method enables implementations that
	 * require the input of auxiliary data to process the operation.
	 * 
	 * @param capturedBIR
	 *            the captured BIR
	 * @param auxiliaryBIRs
	 *            a BIR containing the auxiliary data used in the operation
	 * @param outputFormat
	 *            specifies which BDB format to use for the returned processed
	 *            BIR, if the BSP supports more than one format. null indicates
	 *            that the BSP is to select the format
	 * @return The BIR object that represents the result of processing
	 * @throws BioAPIException
	 *             If the input parameters are invalid or operation fails due to
	 *             error.
	 */
	BIR process(BIR capturedBIR, Vector<BIR> auxiliaryBIRs,
			DataFormat outputFormat) throws BioAPIException;

}
