package org.bioapi;

import java.util.Vector;
import org.bioapi.data.BIR;
import org.bioapi.data.DataFormat;
import org.bioapi.data.DataTypes.BIRPurpose;
import org.bioapi.data.DataTypes.BiometricSubtype;
import org.bioapi.data.DataTypes.ResultOptions;
import org.bioapi.data.DataTypes.UnitIndicatorStatus;
import org.bioapi.data.DataTypes.UnitPowerMode;
import org.bioapi.data.UnitSchema;

/**
 * Represents the group of biometric operations that are available when the unit
 * of SENSOR category is linked to the attach session.
 * 
 * @author See CREDITS.txt
 */
public interface Sensor {

	UnitSchema getUnitSchema();

	void setUnitSchema(UnitSchema unitSchema);

	byte[] getACBioInstance();

	byte[] getAuxiliaryData();

	UnitIndicatorStatus getIndicatorStatus();

	/**
	 * Performs a calibration of the attached sensor if the sensor supports it
	 * 
	 * @param timeout
	 *            The integer value specifying the timeout value in
	 *            milliseconds. -1 means the BSPs default value will be used
	 * @throws BioAPIException
	 *             If the input parameters are invalid or operation fails due to
	 *             error.
	 */
	void calibrate(int timeout) throws BioAPIException;

	/**
	 * Captures samples for the purpose specified, and the BSP returns either an
	 * intermediate type BIR or a processed BIR. The purpose is recorded in the
	 * header of the captured BIR. If RequestAuditData option is specified a BIR
	 * of type raw may be returned in CaptureResult. The BSP is responsible for
	 * serializing
	 * 
	 * @param purpose
	 *            indicates the purpose of the biometric data capture.
	 * @param subtype
	 *            indicates the subtype of the biometric sample acquired. Null
	 *            value indicates value is not provided
	 * @param outputFormat
	 *            Specifies which BDB format to use for the returned processed
	 *            BIR, if the BSP supports more than one format. A null value
	 *            indicates that the BSP is to select the format
	 * @param timeout
	 *            integer value specifying the timeout value in milliseconds.
	 * @param options
	 *            requests additional output such as audit data
	 * @return The BIR object that represents the result of the capture
	 *         operation
	 * @throws BioAPIException
	 *             If the sensor device is busy or operation failed.
	 */
	BIR capture(Vector<BIRPurpose> purpose, BiometricSubtype subtype,
			DataFormat outputFormat, int timeout, Vector<ResultOptions> options)
			throws BioAPIException;

	/**
	 * Sends control data to the BioAPI_Unit and receives status or operation
	 * data from there. The content of the parameters and the output will be
	 * specified in the related interface specification of this BioAPI_Unit
	 * 
	 * @param controlCode
	 *            The function code in the BioAPI_Unit to be called
	 * @param inputData
	 *            Buffer containing the data to be sent to the BioAPI unit
	 *            related to the given ControlCode
	 * @return Data buffer containing the data received from the BioAPI_Unit
	 *         after processing the function indicated by the ControlCode. If no
	 *         memory block has been allocated by the function the value shall
	 *         be NULL.
	 * @throws BioAPIException
	 *             If case of any error
	 */
	byte[] controlUnit(int controlCode, byte[] inputData)
			throws BioAPIException;

	/**
	 * This function sets the selected BioAPI Unit to the requested indicator
	 * status if the BioAPI Unit supports it. After Accept or Reject is set in
	 * the IndicatorStatus parameter, the status will not be changed until the
	 * application sets another value.
	 * 
	 * @param indicatorStatus
	 *            A value to which to set the indicator status of the
	 *            BioAPI_Unit.
	 * @throws BioAPIException
	 *             If case of any error
	 */
	void setIndicatorStatus(UnitIndicatorStatus indicatorStatus)
			throws BioAPIException;

	/**
	 * This function sets the currently attached BioAPI Unit of the referenced
	 * BSP attach session to the requested power mode if the BioAPI Unit
	 * supports it.
	 * 
	 * @param powerMode
	 *            The indicator of the power mode to set the BioAPI_Unit to.
	 * @throws BioAPIException
	 *             If case of any error
	 */
	void setPowerMode(UnitPowerMode powerMode) throws BioAPIException;

}
