package org.bioapi.data;

import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer;
import java.util.Vector;
import org.bioapi.BioAPIException;
import org.bioapi.data.DataTypes.BIRPurpose;
import org.bioapi.data.DataTypes.BiometricSubtype;
import org.bioapi.data.DataTypes.BiometricType;
import org.bioapi.data.DataTypes.Facility;
import org.bioapi.data.DataTypes.ProcessedLevel;
import android.annotation.SuppressLint;
import android.util.Log;

/**
 * Represents BIR. No methods to modify the existing BIR object are provided. An
 * application should use the corresponding method of the DataFactory to create
 * new BIR containing modified data. Depending on the way the BIR object was
 * created, the BIR is either bound to the BSP or is unbound. Output of
 * biometric operation normally creates the BIR bound to the BSP. Bound BIR is
 * the BIR encapsulated in the BSP with the lifetime limited by the lifetime of
 * the AttachSession that created it. The bound BIR can be extracted from a BSP:
 * -implicitly, by calling either of properties BiometricData or SecurityBlock.
 * -explicitly, by calling the Unbind() method.
 * 
 * @author See CREDITS.txt
 */
public class BIR {

	/*
	 * NOTE (daniel s.): This BIR implementation is intended to be consistent
	 * with the 19785 (CBEFF) standard. At this moment, this specification is
	 * under revision so the changes proposed here may be in conflict with the
	 * final release
	 */

	private DataFormat patronFormat;
	private byte cbeffVersion;
	private byte patronHeaderVersion;
	private DataFormat bdbFormat;
	private boolean bdbEncryption;
	private boolean bdbIntegrity;
	private DataTypes.BiometricType bdbBiometricType;
	private DataTypes.BiometricSubtype bdbBiometricSubtype;
	private byte[] bdbChallengeResponse;
	private Date bdbCreationDate;
	private byte[] bdbIndex;
	private DataTypes.ProcessedLevel bdbProcessedLevel;
	private DataFormat bdbProduct;
	private DataTypes.BIRPurpose bdbPurpose;
	private byte bdbQuality;
	private Vector<Date> bdbValidityPeriod;
	private Date birCreationDate;
	private byte[] birCreator;
	private byte[] birIndex;
	private byte[] birPayload;
	private Vector<Date> birValidityPeriod;
	private DataFormat sbFormat;
	private byte[] bdbData;
	private byte[] sbData;

	// New attributes introduced in JP6
	private DataFormat bdbQualityAlgorithm;
	private DataFormat bdbCaptureDevice;
	private DataFormat bdbFeatureExtractionAlgorithm;
	private DataFormat bdbComparisonAlgorithm;
	private DataFormat bdbCompressionAlgorithm;

	private int fieldPresence;
	private int birLength;

	public BIR() {
		// Default values for a new BIR
		patronFormat = new DataFormat((short) 257, (short) 6); // Owner and type
																// defined in
																// 19785-3.5
		cbeffVersion = 0x21; // Version 2.1, proposed by JP4 for 19795-3.5
		patronHeaderVersion = 0x01; // Fixed patron header version in 19785-3.5
		bdbFormat = new DataFormat((short) 0, (short) 0); // Dafault BDB owner
															// and type
		bdbEncryption = false;
		bdbIntegrity = false;
		bdbBiometricType = null;
		bdbBiometricSubtype = null;
		bdbChallengeResponse = null;
		bdbCreationDate = null;
		bdbIndex = null;
		bdbProcessedLevel = null;
		bdbProduct = null;
		bdbPurpose = null;
		bdbQuality = -1;
		bdbValidityPeriod = null;
		birCreationDate = null;
		birCreator = null;
		birIndex = null;
		birPayload = null;
		birValidityPeriod = null;
		sbFormat = null;
		bdbData = null;
		sbData = null;

		// New attributes introduced in JP6
		bdbQualityAlgorithm = null;
		bdbCaptureDevice = null;
		bdbFeatureExtractionAlgorithm = null;
		bdbComparisonAlgorithm = null;
		bdbCompressionAlgorithm = null;
	}

	public DataFormat getPatronFormat() {
		return patronFormat;
	}

	public void setPatronFormat(DataFormat patronFormat) {
		this.patronFormat = patronFormat;
	}

	public byte getPatronHeaderVersion() {
		return patronHeaderVersion;
	}

	public void setPatronHeaderVersion(byte patronHeaderVersion) {
		this.patronHeaderVersion = patronHeaderVersion;
	}

	public byte getCBEFFVersion() {
		return cbeffVersion;
	}

	public void setCBEFFVersion(byte cbeffVersion) {
		this.cbeffVersion = cbeffVersion;
	}

	public DataFormat getBDBFormat() {
		return bdbFormat;
	}

	public void setBDBFormat(DataFormat bdbFormat) {
		this.bdbFormat = bdbFormat;
	}

	public boolean isBDBEncryption() {
		return bdbEncryption;
	}

	public void setBDBEncryption(boolean bdbEncription) {
		this.bdbEncryption = bdbEncription;
	}

	public boolean isBDBIntegrity() {
		return bdbIntegrity;
	}

	public void setBDBIntegrity(boolean bdbIntegrity) {
		this.bdbIntegrity = bdbIntegrity;
	}

	public BiometricType getBDBBiometricType() {
		return bdbBiometricType;
	}

	public void setBDBBiometricType(BiometricType bdbBiometricType) {
		this.bdbBiometricType = bdbBiometricType;
	}

	public BiometricSubtype getBDBBiometricSubtype() {
		return bdbBiometricSubtype;
	}

	public void setBDBBiometricSubtype(BiometricSubtype bdbbiometricSubtype) {
		this.bdbBiometricSubtype = bdbbiometricSubtype;
	}

	public byte[] getBDBChallengeResponse() {
		return bdbChallengeResponse;
	}

	public void setBDBChallengeResponse(byte[] bdbChallengeResponse) {
		this.bdbChallengeResponse = bdbChallengeResponse;
	}

	public Date getBDBCreationDate() {
		return bdbCreationDate;
	}

	public void setBDBCreationDate(Date bdbCreationDate) {
		this.bdbCreationDate = bdbCreationDate;
	}

	public byte[] getBDBIndex() {
		return bdbIndex;
	}

	public void setBDBIndex(byte[] bdbIndex) {
		this.bdbIndex = bdbIndex;
	}

	public ProcessedLevel getBDBProcessedLevel() {
		return bdbProcessedLevel;
	}

	public void setBDBProcessedLevel(ProcessedLevel bdbProcessedLevel) {
		this.bdbProcessedLevel = bdbProcessedLevel;
	}

	public DataFormat getBDBProduct() {
		return bdbProduct;
	}

	public void setBDBProduct(DataFormat bdbProduct) {
		this.bdbProduct = bdbProduct;
	}

	public BIRPurpose getBDBPurpose() {
		return bdbPurpose;
	}

	public void setBDBPurpose(BIRPurpose bdbPurpose) {
		this.bdbPurpose = bdbPurpose;
	}

	public byte getBDBQuality() {
		return bdbQuality;
	}

	public void setBDBQuality(byte bdbQuality) {
		this.bdbQuality = bdbQuality;
	}

	public Vector<Date> getBDBValidityPeriod() {
		return bdbValidityPeriod;
	}

	public void setBDBValidityPeriod(Vector<Date> bdbValidityPeriod) {
		this.bdbValidityPeriod = bdbValidityPeriod;
	}

	public Date getBIRCreationDate() {
		return birCreationDate;
	}

	public void setBIRCreationDate(Date birCreationDate) {
		this.birCreationDate = birCreationDate;
	}

	public byte[] getBIRCreator() {
		return birCreator;
	}

	public void setBIRCreator(byte[] birCreator) {
		this.birCreator = birCreator;
	}

	public byte[] getBIRIndex() {
		return birIndex;
	}

	public void setBIRIndex(byte[] birIndex) {
		this.birIndex = birIndex;
	}

	public byte[] getBIRPayload() {
		return birPayload;
	}

	public void setBIRPayload(byte[] birPayload) {
		this.birPayload = birPayload;
	}

	public Vector<Date> getBIRValidityPeriod() {
		return birValidityPeriod;
	}

	public void setBIRValidityPeriod(Vector<Date> birValidityPeriod) {
		this.birValidityPeriod = birValidityPeriod;
	}

	public DataFormat getSBFormat() {
		return sbFormat;
	}

	public void setSBFormat(DataFormat sbFormat) {
		this.sbFormat = sbFormat;
	}

	public byte[] getBDBData() {
		return bdbData;
	}

	public void setBDBData(byte[] bdbData) {
		this.bdbData = bdbData;
	}

	public byte[] getSBData() {
		return sbData;
	}

	public void setSBData(byte[] sbData) {
		this.sbData = sbData;
	}

	/**
	 * Fills in the BIR data from a byte array coded as a ISO/IEC 19785 record
	 * 
	 * @param record
	 *            the byte array containing the CBEFF record
	 * @throws BioAPIException
	 *             If the input parameters are invalid, the format is not
	 *             supported or operation fails due to error.
	 */
	public void birFromByteArray(byte[] record) throws BioAPIException {
		int fieldPresence;
		ByteBuffer buffer = ByteBuffer.wrap(record);

		// Mandatory fields
		this.setPatronFormat(new DataFormat(buffer.getShort(), buffer
				.getShort()));
		birLength = buffer.getInt();
		this.setCBEFFVersion(buffer.get());
		this.setPatronHeaderVersion(buffer.get());
		fieldPresence = buffer.getInt();
		Log.i("field presence", Integer.toBinaryString(fieldPresence));
		this.setBDBFormat(new DataFormat(buffer.getShort(), buffer.getShort()));
		this.setBDBEncryption((buffer.get() == (byte) 0x00) ? false : true);
		this.setBDBIntegrity((buffer.get() == (byte) 0x00) ? false : true);

		// Optional fields
		if (isFieldPresent(fieldPresence, 3)) { // bdbBiometricType
			int biometricTypeValue = ((0x00 << 24))
					| ((buffer.get() << 16) & 0x00FF0000)
					| ((buffer.get() << 8) & 0x0000FF00)
					| ((buffer.get()) & 0x000000FF);
			BiometricType biometricType = BiometricType
					.getType(biometricTypeValue);
			if (biometricType == null) {
				throw new BioAPIException(Facility.BSP,
						BioAPIException.bioAPIErrInvalidBir,
						"The biometric type is not recognized");
			}
			this.bdbBiometricType = biometricType;
		}
		if (isFieldPresent(fieldPresence, 4)) { // bdbBiometricSubtype
			BiometricSubtype biometricSubtype = BiometricSubtype.getType(buffer
					.get());
			if (biometricSubtype == null) {
				throw new BioAPIException(Facility.BSP,
						BioAPIException.bioAPIErrInvalidBir,
						"The biometric subtype is not recognized");
			}
			this.bdbBiometricSubtype = biometricSubtype;
		}
		if (isFieldPresent(fieldPresence, 5)) { // bdbChallengeResponse
			int challengeLength = buffer.getShort() & 0x0000FFFF;
			this.bdbChallengeResponse = new byte[challengeLength];
			buffer.get(this.bdbChallengeResponse, 0, challengeLength);
		}
		if (isFieldPresent(fieldPresence, 6)) { // bdbCreationDate
			int creationDateLength = buffer.get() & 0x000000FF;
			if (creationDateLength < 8 || creationDateLength > 15) {
				throw new BioAPIException(Facility.BSP,
						BioAPIException.bioAPIErrInvalidBir,
						"Creation date length must be between 8 and 15 bytes");
			} else {
				byte[] dateArray = new byte[creationDateLength];
				buffer.get(dateArray, 0, creationDateLength);
				this.bdbCreationDate = fillDate(dateArray);
			}
		}
		if (isFieldPresent(fieldPresence, 7)) { // bdbIndex
			int indexLength = buffer.getShort() & 0x000000FF;
			this.bdbIndex = new byte[indexLength];
			buffer.get(this.bdbIndex, 0, indexLength);
		}
		if (isFieldPresent(fieldPresence, 8)) { // bdbProcessedLevel
			byte processedLevel = buffer.get();
			switch (processedLevel) {
			case 1:
				this.bdbProcessedLevel = ProcessedLevel.RAW;
				break;
			case 2:
				this.bdbProcessedLevel = ProcessedLevel.INTERMEDIATE;
				break;
			case 3:
				this.bdbProcessedLevel = ProcessedLevel.PROCESSED;
				break;
			default:
				throw new BioAPIException(Facility.BSP,
						BioAPIException.bioAPIErrInvalidBir,
						"Processed level attached to the BIR does not exist");
			}
		}
		if (isFieldPresent(fieldPresence, 9)) { // bdbProduct owner and type
			short owner = buffer.getShort();
			short type = buffer.getShort();
			this.bdbProduct = new DataFormat(owner, type);
		}
		if (isFieldPresent(fieldPresence, 10)) { // bdbPurpose
			byte purpose = buffer.get();
			switch (purpose) {
			case 1:
				this.bdbPurpose = BIRPurpose.VERIFY;
				break;
			case 2:
				this.bdbPurpose = BIRPurpose.IDENTIFY;
				break;
			case 3:
				this.bdbPurpose = BIRPurpose.ENROL;
				break;
			case 4:
				this.bdbPurpose = BIRPurpose.ENROL_FOR_VERIFICATION_ONLY;
				break;
			case 5:
				this.bdbPurpose = BIRPurpose.ENROL_FOR_IDENTIFICATION_ONLY;
				break;
			case 6:
				this.bdbPurpose = BIRPurpose.AUDIT;
				break;
			default:
				this.bdbPurpose = BIRPurpose.NO_PURPOSE_AVAILABLE;
			}
		}
		if (isFieldPresent(fieldPresence, 11)) { // bdbQuality
			this.bdbQuality = buffer.get();
		}
		if (isFieldPresent(fieldPresence, 12)) { // bdbValidityPeriod
			int validityPeriodLength = buffer.get() & 0x000000FF;
			if (validityPeriodLength < 17 || validityPeriodLength > 31) {
				throw new BioAPIException(Facility.BSP,
						BioAPIException.bioAPIErrInvalidBir,
						"Creation date length must be between 17 and 31 bytes");
			}
			byte[] validityPeriodArray = new byte[validityPeriodLength];
			buffer.get(validityPeriodArray, 0, validityPeriodLength);
			String str;
			String[] strArray;
			Date startDate, endDate;
			try {
				str = new String(validityPeriodArray, "UTF-8");
				strArray = str.split("/");
				if (strArray != null && strArray.length == 2) {
					startDate = fillDate(strArray[0].getBytes());
					endDate = fillDate(strArray[1].getBytes());
				} else {
					throw new BioAPIException(Facility.BSP,
							BioAPIException.bioAPIErrInvalidBir,
							"Malformed validity period for the BIR");
				}
			} catch (UnsupportedEncodingException e) {
				throw new BioAPIException(Facility.BSP,
						BioAPIException.bioAPIBspError, "Internal error");
			}
			this.bdbValidityPeriod = new Vector<Date>();
			this.bdbValidityPeriod.add(startDate);
			this.bdbValidityPeriod.add(endDate);
		}
		if (isFieldPresent(fieldPresence, 13)) { // birCreationDate
			int creationDateLength = buffer.get() & 0x000000FF;
			if (creationDateLength < 8 || creationDateLength > 15) {
				throw new BioAPIException(Facility.BSP,
						BioAPIException.bioAPIErrInvalidBir,
						"Creation date length must be between 8 and 15 bytes");
			} else {
				byte[] dateArray = new byte[creationDateLength];
				buffer.get(dateArray, 0, creationDateLength);
				this.birCreationDate = fillDate(dateArray);
			}
		}
		if (isFieldPresent(fieldPresence, 14)) { // birCreator
			int creatorLength = buffer.getShort() & 0x000000FF;
			this.birCreator = new byte[creatorLength];
			buffer.get(this.birCreator, 0, creatorLength);
		}
		if (isFieldPresent(fieldPresence, 15)) { // bdbIndex
			int indexLength = buffer.getShort() & 0x000000FF;
			this.birIndex = new byte[indexLength];
			buffer.get(this.birIndex, 0, indexLength);
		}
		if (isFieldPresent(fieldPresence, 16)) { // birPayload
			int payloadLength = buffer.getShort() & 0x000000FF;
			this.birPayload = new byte[payloadLength];
			buffer.get(this.birPayload, 0, payloadLength);
		}
		if (isFieldPresent(fieldPresence, 17)) { // birValidityPeriod
			int validityPeriodLength = buffer.get() & 0x000000FF;
			if (validityPeriodLength < 17 || validityPeriodLength > 31) {
				throw new BioAPIException(Facility.BSP,
						BioAPIException.bioAPIErrInvalidBir,
						"Creation date length must be between 17 and 31 bytes");
			}
			byte[] validityPeriodArray = new byte[validityPeriodLength];
			String str;
			String[] strArray;
			Date startDate, endDate;
			try {
				str = new String(validityPeriodArray, "UTF-8");
				strArray = str.split("/");
				if (strArray != null && strArray.length == 2) {
					startDate = fillDate(strArray[0].getBytes());
					endDate = fillDate(strArray[1].getBytes());
				} else {
					throw new BioAPIException(Facility.BSP,
							BioAPIException.bioAPIErrInvalidBir,
							"Malformed validity period for the BIR");
				}
			} catch (UnsupportedEncodingException e) {
				throw new BioAPIException(Facility.BSP,
						BioAPIException.bioAPIBspError, "Internal error");
			}
			this.birValidityPeriod = new Vector<Date>();
			this.birValidityPeriod.add(startDate);
			this.birValidityPeriod.add(endDate);
		}
		if (isFieldPresent(fieldPresence, 18)) { // sb owner and type
			if (!isFieldPresent(fieldPresence, 20)) {
				throw new BioAPIException(Facility.BSP,
						BioAPIException.bioAPIErrInvalidBir,
						"No SB data found! SB format and type must not appear in field presence field");
			} else {
				short owner = buffer.getShort();
				short type = buffer.getShort();
				this.sbFormat.setOwner(owner);
				this.sbFormat.setType(type);
			}
		}
		if (isFieldPresent(fieldPresence, 19)) { // bdb data
			int bdbLength = buffer.getInt();
			this.bdbData = new byte[bdbLength];
			buffer.get(this.bdbData, 0, bdbLength);
		}
		if (isFieldPresent(fieldPresence, 20)) { // sb data
			int sbLength = buffer.getInt();
			this.sbData = new byte[sbLength];
			buffer.get(this.sbData, 0, sbLength);
		}
		if (isFieldPresent(fieldPresence, 21)) { // bdbQualityAlgorithm owner
													// and type
			short owner = buffer.getShort();
			short type = buffer.getShort();
			this.bdbQualityAlgorithm.setOwner(owner);
			this.bdbQualityAlgorithm.setType(type);
		}
		if (isFieldPresent(fieldPresence, 22)) { // bdbCaptureDevice owner and
													// type
			short owner = buffer.getShort();
			short type = buffer.getShort();
			this.bdbCaptureDevice.setOwner(owner);
			this.bdbCaptureDevice.setType(type);
		}
		if (isFieldPresent(fieldPresence, 23)) { // bdbExtractionAlgorithm owner
													// and type
			short owner = buffer.getShort();
			short type = buffer.getShort();
			this.bdbFeatureExtractionAlgorithm.setOwner(owner);
			this.bdbFeatureExtractionAlgorithm.setType(type);
		}
		if (isFieldPresent(fieldPresence, 24)) { // bdbComparisonAlgorithm owner
													// and type
			short owner = buffer.getShort();
			short type = buffer.getShort();
			this.bdbComparisonAlgorithm.setOwner(owner);
			this.bdbComparisonAlgorithm.setType(type);
		}
		if (isFieldPresent(fieldPresence, 25)) { // bdbCompressionAlgorithm
													// owner and type
			short owner = buffer.getShort();
			short type = buffer.getShort();
			this.bdbCompressionAlgorithm.setOwner(owner);
			this.bdbCompressionAlgorithm.setType(type);
		}
	}

	/*
	 * This method create a new Date object from byte array represented by ISO
	 * 8601 standard
	 */
	private Date fillDate(byte[] dateArray) throws BioAPIException {
		ByteBuffer buffer = ByteBuffer.wrap(dateArray);
		int year = 0, month = 0, day = 0, hour = 0, minute = 0, second = 0;
		boolean error = false;

		year = buffer.get() << 24 | buffer.get() << 16 | buffer.get() << 8
				| buffer.get();
		month = buffer.get() << 8 | buffer.get();
		day = buffer.get() << 8 | buffer.get();
		if (dateArray.length >= 11) {
			buffer.get(); // T
			hour = buffer.get() << 8 | buffer.get();
			if (dateArray.length >= 13) {
				minute = buffer.get() << 8 | buffer.get();
				if (dateArray.length == 15) {
					second = buffer.get() << 8 | buffer.get();
				} else {
					error = true;
				}
				;
			} else {
				error = true;
			}
		} else {
			error = true;
		}

		if (error) {
			throw new BioAPIException(Facility.BSP,
					BioAPIException.bioAPIErrInvalidBir,
					"Invalid format for ISO 8601 date array");
		} else {
			return new Date(day, month, year, hour, minute, second);
		}
	}

	/*
	 * This method evaluate the presence of the field identified by
	 * 'fieldNumber' in the 'filedPresence' array. The correspondence between
	 * the fieldNumber and the field can be found in 19785-3 specification
	 */
	private boolean isFieldPresent(int fieldPresence, int fieldNumber) {
		if ((((fieldPresence >> (32 - fieldNumber)) & 1) == 1)) {
			// The field is present
			return true;
		} else {
			// The field is not present
			return false;
		}
	}

	/**
	 * Serializes a BIR record so as to provide it as a byte array representing
	 * the CBEFF information
	 * 
	 * @return The byte array containing the CBEFF information
	 * @throws BioAPIException
	 *             If the input parameters are invalid, the format is not
	 *             supported or operation fails due to error.
	 */
	public byte[] birToByteArray() throws BioAPIException {
		refreshBIRLengthAndFieldPresence();
		int birLength = this.birLength;
		int fieldPresence = this.fieldPresence;
		Log.i("FIELD_PRESENCE", Integer.toHexString(fieldPresence));
		byte[] birArray = new byte[birLength];
		int pointer = 0;

		// Patron format
		if (this.patronFormat != null && this.patronFormat.getOwner() == 257
				&& this.patronFormat.getType() == 6) {
			byte[] array = this.patronFormat.toByteArray();
			System.arraycopy(array, 0, birArray, pointer, array.length);
			pointer += array.length;
		} else {
			throw new BioAPIException(Facility.BSP,
					BioAPIException.bioAPIErrInvalidBir,
					"Patron format owner and type must be 257 and 6, respectively");
		}

		// BIR length
		System.arraycopy(ByteBuffer.allocate(4).putInt(birLength).array(), 0,
				birArray, pointer, 4);
		pointer += 4;

		// CBEFF version
		if (this.cbeffVersion == 0x21) {
			birArray[pointer++] = this.cbeffVersion;
		} else {
			throw new BioAPIException(Facility.BSP,
					BioAPIException.bioAPIErrInvalidBir,
					"CBEFF version must have 0x21 value");
		}

		// Patron header version
		if (this.patronHeaderVersion == 0x01) {
			birArray[pointer++] = this.patronHeaderVersion;
		} else {
			throw new BioAPIException(Facility.BSP,
					BioAPIException.bioAPIErrInvalidBir,
					"Patron header version must have 0x01 value");
		}

		// fieldPresence
		System.arraycopy(ByteBuffer.allocate(4).putInt(fieldPresence).array(),
				0, birArray, pointer, 4);
		pointer += 4;

		// BDB format
		if (this.bdbFormat != null) {
			byte[] array = this.bdbFormat.toByteArray();
			System.arraycopy(array, 0, birArray, pointer, array.length);
			pointer += array.length;
		} else {
			// BDB owner and format to 0 by default
			byte[] array = (new DataFormat((byte) 0x00, (byte) 0x00))
					.toByteArray();
			System.arraycopy(array, 0, birArray, pointer, array.length);
			pointer += array.length;
		}

		// BDB Encryption
		if (this.bdbEncryption == false) {
			birArray[pointer++] = 0;
		} else {
			birArray[pointer++] = 1;
		}

		// BDB Integrity
		if (this.bdbIntegrity == false) {
			birArray[pointer++] = 0;
		} else {
			birArray[pointer++] = 1;
		}

		// BDB Biometric Type
		if (this.bdbBiometricType != null) {
			byte[] array = ByteBuffer.allocate(4)
					.putInt(this.bdbBiometricType.getValue()).array();
			System.arraycopy(array, 1, birArray, pointer, 3);
			pointer += 3;
		}

		// BDB Biometric Type
		if (this.bdbBiometricSubtype != null) {
			birArray[pointer++] = this.bdbBiometricSubtype.getValue();
		}

		// BDB challenge response
		if (this.bdbChallengeResponse != null) {
			short length = (short) (this.bdbChallengeResponse.length & 0x0000FFFF);
			System.arraycopy(ByteBuffer.allocate(2).putShort(length).array(),
					0, birArray, pointer, 2);
			pointer += 2;
			System.arraycopy(this.bdbChallengeResponse, 0, birArray, pointer,
					length);
			pointer += length;
		}

		// BDB creation date
		if (this.bdbCreationDate != null) {
			byte[] array = getDateArray(this.bdbCreationDate);
			birArray[pointer++] = (byte) (array.length & 0x000000FF);
			System.arraycopy(getDateArray(this.bdbCreationDate), 0, birArray,
					pointer, array.length);
			pointer += array.length;
		}

		// BDB index
		if (this.bdbIndex != null) {
			short length = (short) (this.bdbIndex.length & 0x0000FFFF);
			System.arraycopy(ByteBuffer.allocate(2).putShort(length).array(),
					0, birArray, pointer, 2);
			pointer += 2;
			System.arraycopy(this.bdbIndex, 0, birArray, pointer, length);
			pointer += length;
		}

		// BDB processed level
		if (this.bdbProcessedLevel != null) {
			birArray[pointer++] = this.bdbProcessedLevel.getValue();
		}

		// BDB product
		if (this.bdbProduct != null) {
			byte[] array = this.bdbProduct.toByteArray();
			System.arraycopy(array, 0, birArray, pointer, array.length);
			pointer += array.length;
		}

		// BDB purpose
		if (this.bdbPurpose != null) {
			switch (this.bdbPurpose) {
			case VERIFY:
				birArray[pointer++] = 0x01;
				break;
			case IDENTIFY:
				birArray[pointer++] = 0x02;
				break;
			case ENROL:
				birArray[pointer++] = 0x03;
				break;
			case ENROL_FOR_VERIFICATION_ONLY:
				birArray[pointer++] = 0x04;
				break;
			case ENROL_FOR_IDENTIFICATION_ONLY:
				birArray[pointer++] = 0x05;
				break;
			case AUDIT:
				birArray[pointer++] = 0x06;
				break;
			default:
				Log.w("BDB_PURPOSE_VALUE",
						"The value is not consistent with CBEFF standard");
			}
		}

		// BDB quality
		if (this.bdbQuality > 100 && this.bdbQuality < 254) {
			birArray[pointer++] = this.bdbQuality;
		}

		// BDB validity period
		if (this.bdbValidityPeriod != null) {
			if (this.bdbValidityPeriod.size() == 2) {
				byte[] array1 = getDateArray(this.bdbValidityPeriod.get(0));
				byte[] array2 = getDateArray(this.bdbValidityPeriod.get(1));
				birArray[pointer++] = (byte) ((array1.length + array2.length + 1) & 0x000000FF);
				System.arraycopy(array1, 0, birArray, pointer, array1.length);
				pointer += array1.length;
				birArray[pointer++] = '/';
				System.arraycopy(array2, 0, birArray, pointer, array2.length);
				pointer += array2.length;
			} else {
				throw new BioAPIException(Facility.BSP,
						BioAPIException.bioAPIErrInvalidBir,
						"The length of validity period vector must be 2");
			}
		}

		// BIR creation date
		if (this.birCreationDate != null) {
			byte[] array = getDateArray(this.birCreationDate);
			birArray[pointer++] = (byte) (array.length & 0x000000FF);
			System.arraycopy(array, 0, birArray, pointer, array.length);
			pointer += array.length;
		}

		// BIR creator
		if (this.birCreator != null) {
			short length = (short) (this.birCreator.length & 0x0000FFFF);
			System.arraycopy(ByteBuffer.allocate(2).putShort(length).array(),
					0, birArray, pointer, 2);
			pointer += 2;
			System.arraycopy(this.birCreator, 0, birArray, pointer, length);
			pointer += length;
		}

		// BIR index
		if (this.birIndex != null) {
			short length = (short) (this.birIndex.length & 0x0000FFFF);
			System.arraycopy(ByteBuffer.allocate(2).putShort(length).array(),
					0, birArray, pointer, 2);
			pointer += 2;
			System.arraycopy(this.birIndex, 0, birArray, pointer, length);
			pointer += length;
		}

		// BIR payload
		if (this.birPayload != null) {
			short length = (short) (this.birPayload.length & 0x0000FFFF);
			System.arraycopy(ByteBuffer.allocate(2).putShort(length).array(),
					0, birArray, pointer, 2);
			pointer += 2;
			System.arraycopy(this.birPayload, 0, birArray, pointer, length);
			pointer += length;
		}

		// BIR validity period
		if (this.birValidityPeriod != null) {
			if (this.birValidityPeriod.size() == 2) {
				byte[] array1 = getDateArray(this.birValidityPeriod.get(0));
				byte[] array2 = getDateArray(this.birValidityPeriod.get(1));
				birArray[pointer++] = (byte) ((array1.length + array2.length + 1) & 0x000000FF);
				System.arraycopy(array1, 0, birArray, pointer, array1.length);
				pointer += array1.length;
				birArray[pointer++] = '/';
				System.arraycopy(array2, 0, birArray, pointer, array2.length);
				pointer += array2.length;
			} else {
				throw new BioAPIException(Facility.BSP,
						BioAPIException.bioAPIErrInvalidBir,
						"The length of validity period vector must be 2");
			}
		}

		// SB format
		if (this.sbFormat != null) {
			if (this.sbData != null) {
				byte[] array = this.sbFormat.toByteArray();
				System.arraycopy(array, 0, birArray, pointer, array.length);
				pointer += array.length;
			} else {
				throw new BioAPIException(Facility.BSP,
						BioAPIException.bioAPIErrInvalidBir,
						"The SB format is present but SB data was not found");
			}
		}

		// BDB data
		if (this.bdbData != null) {
			int length = this.bdbData.length;
			System.arraycopy(ByteBuffer.allocate(4).putInt(length).array(), 0,
					birArray, pointer, 4);
			pointer += 4;
			System.arraycopy(this.bdbData, 0, birArray, pointer, length);
			pointer += length;
		}

		// SB data
		if (this.sbData != null) {
			int length = this.sbData.length;
			System.arraycopy(ByteBuffer.allocate(4).putInt(length).array(), 0,
					birArray, pointer, 4);
			pointer += 4;
			System.arraycopy(this.sbData, 0, birArray, pointer, length);
			pointer += length;
		}

		// BDB quality algorithm
		if (this.bdbQualityAlgorithm != null) {
			byte[] array = this.sbFormat.toByteArray();
			System.arraycopy(array, 0, birArray, pointer, array.length);
			pointer += array.length;
		}

		// BDB capture device
		if (this.bdbCaptureDevice != null) {
			byte[] array = this.sbFormat.toByteArray();
			System.arraycopy(array, 0, birArray, pointer, array.length);
			pointer += array.length;
		}

		// BDB feature extraction algorithm
		if (this.bdbFeatureExtractionAlgorithm != null) {
			byte[] array = this.sbFormat.toByteArray();
			System.arraycopy(array, 0, birArray, pointer, array.length);
			pointer += array.length;
		}

		// BDB comparison algorithm
		if (this.bdbComparisonAlgorithm != null) {
			byte[] array = this.sbFormat.toByteArray();
			System.arraycopy(array, 0, birArray, pointer, array.length);
			pointer += array.length;
		}

		// BDB compresion algorithm
		if (this.bdbCompressionAlgorithm != null) {
			byte[] array = this.sbFormat.toByteArray();
			System.arraycopy(array, 0, birArray, pointer, array.length);
			pointer += array.length;
		}

		return birArray;
	}

	private byte[] getDateArray(Date date) {

		byte[] dayOfMonth = String.format("%02d", date.getDayOfMonth())
				.getBytes();
		byte[] month = String.format("%02d", date.getMonth()).getBytes();
		byte[] year = String.format("%04d", date.getYear()).getBytes();
		byte[] hour = String.format("%02d", date.getHour()).getBytes();
		byte[] minute = String.format("%02d", date.getMinute()).getBytes();
		byte[] second = String.format("%02d", date.getSecond()).getBytes();
		byte T = 'T';

		byte[] array = new byte[15];
		System.arraycopy(year, 0, array, 0, year.length);
		System.arraycopy(month, 0, array, 4, month.length);
		System.arraycopy(dayOfMonth, 0, array, 6, dayOfMonth.length);
		array[8] = T;
		System.arraycopy(hour, 0, array, 9, hour.length);
		System.arraycopy(minute, 0, array, 11, minute.length);
		System.arraycopy(second, 0, array, 13, second.length);

		return array;
	}

	private void refreshBIRLengthAndFieldPresence() {
		int length = 0;
		int fieldPresence = 0;
		length += 20; // Fixed length for mandatory fields
		fieldPresence |= 0xC0000000; // Fixed field presence for mandatory
										// fields
		if (this.bdbBiometricType != null) {
			length += 3;
			fieldPresence |= 0x20000000;
		}
		if (this.bdbBiometricSubtype != null) {
			length++;
			fieldPresence |= 0x10000000;
		}
		if (this.bdbChallengeResponse != null) {
			length += (2 + this.bdbChallengeResponse.length);
			fieldPresence |= 0x08000000;
		}
		if (this.bdbCreationDate != null) {
			length += (1 + 15);
			fieldPresence |= 0x04000000;
		}
		if (this.bdbIndex != null) {
			length += (2 + bdbIndex.length);
			fieldPresence |= 0x02000000;
		}
		if (this.bdbProcessedLevel != null) {
			length++;
			fieldPresence |= 0x01000000;
		}
		if (this.bdbProduct != null) {
			length += 4;
			fieldPresence |= 0x00800000;
		}
		if (this.bdbPurpose != null) {
			length++;
			fieldPresence |= 0x00400000;
		}
		if (this.bdbQuality > 100 && this.bdbQuality < 254) {
			length++;
			fieldPresence |= 0x00200000;
		}
		if (this.bdbValidityPeriod != null) {
			length += (1 + 31);
			fieldPresence |= 0x00100000;
		}
		if (this.birCreationDate != null) {
			length += (1 + 15);
			fieldPresence |= 0x00080000;
		}
		if (this.birCreator != null) {
			length += (2 + this.birCreator.length);
			fieldPresence |= 0x00040000;
		}
		if (this.birIndex != null) {
			length += (2 + this.birIndex.length);
			fieldPresence |= 0x00020000;
		}
		if (this.birPayload != null) {
			length += (2 + this.birPayload.length);
			fieldPresence |= 0x00010000;
		}
		if (this.birValidityPeriod != null) {
			length += (1 + 31);
			fieldPresence |= 0x00008000;
		}
		if (this.sbFormat != null && this.sbData != null) {
			length += 4;
			fieldPresence |= 0x00004000;
		}
		if (this.bdbData != null) {
			length += (4 + this.bdbData.length);
			fieldPresence |= 0x00002000;
		}
		if (this.sbData != null && this.sbFormat != null) {
			length += (4 + this.sbData.length);
			fieldPresence |= 0x00001000;
		}
		if (this.bdbQualityAlgorithm != null) {
			length += 4;
			fieldPresence |= 0x00000800;
		}
		if (this.bdbCaptureDevice != null) {
			length += 4;
			fieldPresence |= 0x00000400;
		}
		if (this.bdbFeatureExtractionAlgorithm != null) {
			length += 4;
			fieldPresence |= 0x00000200;
		}
		if (this.bdbComparisonAlgorithm != null) {
			length += 4;
			fieldPresence |= 0x00000100;
		}
		if (this.bdbCompressionAlgorithm != null) {
			length += 4;
			fieldPresence |= 0x00000080;
		}
		this.birLength = length;
		this.fieldPresence = fieldPresence;
	}

	/**
	 * Removes all the information in the current BIR, leaving it empty for a
	 * next use
	 */
	void destroy() {
		patronFormat = new DataFormat((short) 257, (short) 6);
		cbeffVersion = 0x21;
		patronHeaderVersion = 0x01;
		bdbFormat = new DataFormat((short) 0, (short) 0);
		bdbEncryption = false;
		bdbIntegrity = false;
		bdbBiometricType = null;
		bdbBiometricSubtype = null;
		bdbChallengeResponse = null;
		bdbCreationDate = null;
		bdbIndex = null;
		bdbProcessedLevel = null;
		bdbProduct = null;
		bdbPurpose = null;
		bdbQuality = -1;
		bdbValidityPeriod = null;
		birCreationDate = null;
		birCreator = null;
		birIndex = null;
		birPayload = null;
		birValidityPeriod = null;
		sbFormat = null;
		bdbData = null;
		sbData = null;
		bdbQualityAlgorithm = null;
		bdbCaptureDevice = null;
		bdbFeatureExtractionAlgorithm = null;
		bdbComparisonAlgorithm = null;
		bdbCompressionAlgorithm = null;
	}

	/**
	 * Request about each of the above mentioned characteristics of the BIR
	 * 
	 * @return True if the characteristic is available, False in other case.
	 */
	boolean isBDBEncrypted() {
		return false;
	}

	boolean isBIRSigned() {
		return false;
	}

	boolean isQualitySupported() {
		return false;
	}

	boolean isQualityKnown() {
		return false;
	}

}
